/*!
 * @quasar/quasar-ui-qcalendar v4.1.2
 * (c) 2025 Jeff Galbraith <jeff@quasar.dev>
 * Released under the MIT License.
 */

'use strict';

var vue = require('vue');

const PARSE_DATETIME = /^(\d{4})-(\d{1,2})(-(\d{1,2}))?([^\d]+(\d{1,2}))?(:(\d{1,2}))?(:(\d{1,2}))?(.(\d{1,3}))?$/;
const PARSE_DATE = /^(\d{4})-(\d{1,2})(-(\d{1,2}))?/;
const PARSE_TIME = /(\d\d?)(:(\d\d?)|)(:(\d\d?)|)/;
const DAYS_IN_MONTH = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
const DAYS_IN_MONTH_LEAP = [0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
const TIME_CONSTANTS = {
    MILLISECONDS_IN: {
        SECOND: 1000,
        MINUTE: 60000,
        HOUR: 3600000,
        DAY: 86400000,
        WEEK: 604800000,
    },
    SECONDS_IN: {
        MINUTE: 60,
        HOUR: 3600,
        DAY: 86400,
        WEEK: 604800,
    },
    MINUTES_IN: {
        MINUTE: 1,
        HOUR: 60,
        DAY: 1440,
        WEEK: 10080,
    },
    HOURS_IN: {
        DAY: 24,
        WEEK: 168,
    },
    DAYS_IN: {
        WEEK: 7,
    },
};
const DAYS_IN_MONTH_MIN = 28;
const DAYS_IN_MONTH_MAX = 31;
const MONTH_MAX = 12;
const MONTH_MIN = 1;
const DAY_MIN = 1;
const FIRST_HOUR = 0;
/**
 * Validates the passed input ('YYY-MM-DD') as a date or ('YYY-MM-DD HH:MM') date time combination
 * @param {string} input A string in the form 'YYYY-MM-DD' or 'YYYY-MM-DD HH:MM'
 * @returns {boolean} True if parseable
 */
function validateTimestamp(input) {
    if (typeof input !== 'string')
        return false;
    return PARSE_DATETIME.test(input);
}
/**
 * Fast low-level parser for a date string ('YYYY-MM-DD'). Does not update formatted or relative date.
 * Use 'parseTimestamp' for formatted and relative updates
 * @param {string} input In the form 'YYYY-MM-DD hh:mm:ss' (seconds are optional, but not used)
 * @returns {Timestamp} This {@link Timestamp} is minimally filled in. The {@link Timestamp.date} and {@link Timestamp.time} as well as relative data will not be filled in.
 */
function parsed(input) {
    if (typeof input !== 'string')
        return null;
    const parts = PARSE_DATETIME.exec(input);
    if (!parts || !parts[1] || !parts[2])
        return null;
    const year = parseInt(parts[1], 10);
    const month = parseInt(parts[2], 10);
    const day = parseInt(parts[4] || '1', 10);
    const hour = parseInt(parts[6] || '0', 10);
    const minute = parseInt(parts[8] || '0', 10);
    return {
        date: input,
        time: `${hour.toString().padStart(2, '0')}:${minute.toString().padStart(2, '0')}`,
        year,
        month,
        day,
        hour,
        minute,
        hasDay: !!parts[4],
        hasTime: true, // time is always present, even if '00:00'
        past: false,
        current: false,
        future: false,
        disabled: false,
        weekday: 0,
        doy: 0,
        workweek: 0,
    };
}
/**
 * Takes a JavaScript Date and returns a {@link Timestamp}. The {@link Timestamp} is not updated with relative information.
 * @param {Date} date JavaScript Date
 * @param {boolean} utc If set the {@link Timestamp} will parse the Date as UTC
 * @returns {Timestamp} A minimal {@link Timestamp} without updated or relative updates.
 */
function parseDate(date, utc = false) {
    if (!(date instanceof Date))
        return null;
    const UTC = utc ? 'UTC' : '';
    return updateFormatted({
        date: padNumber(date[`get${UTC}FullYear`](), 4) +
            '-' +
            padNumber(date[`get${UTC}Month`]() + 1, 2) +
            '-' +
            padNumber(date[`get${UTC}Date`](), 2),
        time: padNumber(date[`get${UTC}Hours`]() || 0, 2) +
            ':' +
            padNumber(date[`get${UTC}Minutes`]() || 0, 2),
        year: date[`get${UTC}FullYear`](),
        month: date[`get${UTC}Month`]() + 1,
        day: date[`get${UTC}Date`](),
        hour: date[`get${UTC}Hours`](),
        minute: date[`get${UTC}Minutes`](),
        weekday: 0,
        doy: 0,
        workweek: 0,
        hasDay: true,
        hasTime: true, // Date always has time, even if it is '00:00'
        past: false,
        current: false,
        future: false,
        disabled: false,
    });
}
/**
 * Padds a passed in number to length (converts to a string). Good for converting '5' as '05'.
 * @param {number} x The number to pad
 * @param {number} length The length of the required number as a string
 * @returns {string} The padded number (as a string). (ie: 5 = '05')
 */
function padNumber(x, length) {
    let padded = String(x);
    while (padded.length < length) {
        padded = '0' + padded;
    }
    return padded;
}
/**
 * Returns if the passed year is a leap year
 * @param {number} year The year to check (ie: 1999, 2020)
 * @returns {boolean} True if the year is a leap year
 */
function isLeapYear(year) {
    // A year is a Gregorian leap year if it is divisible by 4,
    // but not by 100, unless it is also divisible by 400.
    return (year % 4 === 0 && year % 100 !== 0) || year % 400 === 0;
}
/**
 * Returns the days of the specified month in a year
 * @param {number} year The year (ie: 1999, 2020)
 * @param {number} month The month (zero-based)
 * @returns {number} The number of days in the month (corrected for leap years)
 */
function daysInMonth(year, month) {
    return (isLeapYear(year) ? DAYS_IN_MONTH_LEAP[month] : DAYS_IN_MONTH[month]);
}
/**
 * Returns a {@link Timestamp} of next day from passed in {@link Timestamp}
 * @param {Timestamp} timestamp The {@link Timestamp} to use
 * @returns {Timestamp} The modified {@link Timestamp} as the next day
 */
function nextDay(timestamp) {
    const date = new Date(timestamp.year, timestamp.month - 1, timestamp.day + 1);
    return updateFormatted(normalizeTimestamp({
        ...timestamp,
        year: date.getFullYear(),
        month: date.getMonth() + 1,
        day: date.getDate(),
    }));
}
/**
 * Returns a {@link Timestamp} of previous day from passed in {@link Timestamp}
 * @param {Timestamp} timestamp The {@link Timestamp} to use
 * @returns {Timestamp} The modified {@link Timestamp} as the previous day
 */
function prevDay(timestamp) {
    const date = new Date(timestamp.year, timestamp.month - 1, timestamp.day - 1);
    return updateFormatted(normalizeTimestamp({
        ...timestamp,
        year: date.getFullYear(),
        month: date.getMonth() + 1,
        day: date.getDate(),
    }));
}
/**
 * Returns today's date
 * @returns {string} Date string in the form 'YYYY-MM-dd'
 */
function today() {
    const d = new Date(), month = d.getMonth() + 1, day = d.getDate(), year = d.getFullYear();
    return [year, padNumber(month, 2), padNumber(day, 2)].join('-');
}
/**
 * Takes a date string ('YYYY-MM-DD') and validates if it is today's date
 * @param {string} date Date string in the form 'YYYY-MM-DD'
 * @returns {boolean} True if the date is today's date
 */
function isToday(date) {
    return date === today();
}
/**
 * Returns the start of the week give a {@link Timestamp} and weekdays (in which it finds the day representing the start of the week).
 * If today {@link Timestamp} is passed in then this is used to update relative information in the returned {@link Timestamp}.
 * @param {Timestamp} timestamp The {@link Timestamp} to use to find the start of the week
 * @param {number[]} weekdays The array is [0,1,2,3,4,5,6] where 0=Sunday and 6=Saturday
 * @param {Timestamp=} today If passed in then the {@link Timestamp} is updated with relative information
 * @returns {Timestamp} The {@link Timestamp} representing the start of the week
 */
function getStartOfWeek(timestamp, weekdays, today) {
    let start = copyTimestamp(timestamp);
    if (!weekdays) {
        return start;
    }
    if (start.day === 1 || start.weekday === 0) {
        while (!weekdays.includes(Number(start.weekday))) {
            start = nextDay(start);
        }
    }
    start = findWeekday(start, weekdays[0], prevDay);
    start = updateFormatted(start);
    if (today) {
        start = updateRelative(start, today, start.hasTime);
    }
    return start;
}
/**
 * Returns the end of the week give a {@link Timestamp} and weekdays (in which it finds the day representing the last of the week).
 * If today {@link Timestamp} is passed in then this is used to update relative information in the returned {@link Timestamp}.
 * @param {Timestamp} timestamp The {@link Timestamp} to use to find the end of the week
 * @param {number[]} weekdays The array is [0,1,2,3,4,5,6] where 0=Sunday and 6=Saturday
 * @param {Timestamp=} today If passed in then the {@link Timestamp} is updated with relative information
 * @returns {Timestamp} The {@link Timestamp} representing the end of the week
 */
function getEndOfWeek(timestamp, weekdays, today) {
    let end = copyTimestamp(timestamp);
    if (!weekdays || !Array.isArray(weekdays)) {
        return end;
    }
    // is last day of month?
    const lastDay = daysInMonth(end.year, end.month);
    if (lastDay === end.day || end.weekday === weekdays[weekdays.length - 1]) {
        while (!weekdays.includes(Number(end.weekday))) {
            end = prevDay(end);
        }
    }
    end = findWeekday(end, weekdays[weekdays.length - 1], nextDay);
    end = updateFormatted(end);
    if (today) {
        end = updateRelative(end, today, end.hasTime);
    }
    return end;
}
/**
 * Finds the start of the month based on the passed in {@link Timestamp}
 * @param {Timestamp} timestamp The {@link Timestamp} to use to find the start of the month
 * @returns {Timestamp} A {@link Timestamp} of the start of the month
 */
function getStartOfMonth(timestamp) {
    let start = copyTimestamp(timestamp);
    start.day = DAY_MIN;
    start = updateFormatted(start);
    return start;
}
/**
 * Finds the end of the month based on the passed in {@link Timestamp}
 * @param {Timestamp} timestamp The {@link Timestamp} to use to find the end of the month
 * @returns {Timestamp} A {@link Timestamp} of the end of the month
 */
function getEndOfMonth(timestamp) {
    let end = copyTimestamp(timestamp);
    end.day = daysInMonth(end.year, end.month);
    end = updateFormatted(end);
    return end;
}
// returns minutes since midnight
function parseTime(input) {
    const type = Object.prototype.toString.call(input);
    switch (type) {
        case '[object Number]':
            // when a number is given, it's minutes since 12:00am
            return input;
        case '[object String]': {
            // when a string is given, it's a hh:mm:ss format where seconds are optional, but not used
            const parts = PARSE_TIME.exec(input);
            if (!parts) {
                return false;
            }
            return parseInt(parts[1], 10) * 60 + parseInt(parts[3] || '0', 10);
        }
        case '[object Object]':
            // when an object is given, it must have hour and minute
            if (typeof input !== 'object' ||
                typeof input.hour !== 'number' ||
                typeof input.minute !== 'number') {
                return false;
            }
            if (typeof input === 'object' && 'hour' in input && 'minute' in input) {
                return input.hour * 60 + input.minute;
            }
            return false;
    }
    return false;
}
/**
 * Compares two {@link Timestamp}s for exactness
 * @param {Timestamp} ts1 The first {@link Timestamp}
 * @param {Timestamp} ts2 The second {@link Timestamp}
 * @returns {boolean} True if the two {@link Timestamp}s are an exact match
 */
function compareTimestamps(ts1, ts2) {
    if (!ts1 || !ts2)
        return false;
    return (ts1.year === ts2.year &&
        ts1.month === ts2.month &&
        ts1.day === ts2.day &&
        ts1.hour === ts2.hour &&
        ts1.minute === ts2.minute);
}
/**
 * Compares the date of two {@link Timestamp}s that have been updated with relative data
 * @param {Timestamp} ts1 The first {@link Timestamp}
 * @param {Timestamp} ts2 The second {@link Timestamp}
 * @returns {boolean} True if the two dates are the same
 */
function compareDate(ts1, ts2) {
    return getDate(ts1) === getDate(ts2);
}
/**
 * Compares the time of two {@link Timestamp}s that have been updated with relative data
 * @param {Timestamp} ts1 The first {@link Timestamp}
 * @param {Timestamp} ts2 The second {@link Timestamp}
 * @returns {boolean} True if the two times are an exact match
 */
function compareTime(ts1, ts2) {
    return getTime(ts1) === getTime(ts2);
}
/**
 * Compares the date and time of two {@link Timestamp}s that have been updated with relative data
 * @param {Timestamp} ts1 The first {@link Timestamp}
 * @param {Timestamp} ts2 The second {@link Timestamp}
 * @returns {boolean} True if the date and time are an exact match
 */
function compareDateTime(ts1, ts2) {
    return getDateTime(ts1) === getDateTime(ts2);
}
/**
 * High-level parser that converts the passed in string to {@link Timestamp} and uses 'now' to update relative information.
 * @param {string} input In the form 'YYYY-MM-DD hh:mm:ss' (seconds are optional, but not used)
 * @param {Timestamp} now A {@link Timestamp} to use for relative data updates
 * @returns {Timestamp} The {@link Timestamp.date} will be filled in as well as the {@link Timestamp.time} if a time is supplied and formatted fields (doy, weekday, workweek, etc). If 'now' is supplied, then relative data will also be updated.
 */
function parseTimestamp(input, now = null) {
    let timestamp = parsed(input);
    if (!timestamp)
        return null;
    timestamp = updateFormatted(timestamp);
    if (now) {
        timestamp = updateRelative(timestamp, now, timestamp.hasTime);
    }
    return timestamp;
}
/**
 * Converts a {@link Timestamp} into a numeric date identifier based on the passed {@link Timestamp}'s date
 * @param {Timestamp} timestamp The {@link Timestamp} to use
 * @returns {number} The numeric date identifier
 */
function getDayIdentifier(timestamp) {
    return ((timestamp.year ?? 0) * 100000000 +
        (timestamp.month ?? 0) * 1000000 +
        (timestamp.day ?? 0) * 10000);
}
/**
 * Converts a {@link Timestamp} into a numeric time identifier based on the passed {@link Timestamp}'s time
 * @param {Timestamp} timestamp The {@link Timestamp} to use
 * @returns {number} The numeric time identifier
 */
function getTimeIdentifier(timestamp) {
    return (timestamp.hour ?? 0) * 100 + (timestamp.minute ?? 0);
}
/**
 * Converts a {@link Timestamp} into a numeric date and time identifier based on the passed {@link Timestamp}'s date and time
 * @param {Timestamp} timestamp The {@link Timestamp} to use
 * @returns {number} The numeric date+time identifier
 */
function getDayTimeIdentifier(timestamp) {
    return getDayIdentifier(timestamp) + getTimeIdentifier(timestamp);
}
/**
 * Returns the difference between two {@link Timestamp}s
 * @param {Timestamp} ts1 The first {@link Timestamp}
 * @param {Timestamp} ts2 The second {@link Timestamp}
 * @param {boolean=} strict Optional flag to not to return negative numbers
 * @returns {number} The difference
 */
function diffTimestamp(ts1, ts2, strict = false) {
    const utc1 = Date.UTC(ts1.year ?? 0, (ts1.month ?? 1) - 1, ts1.day ?? 1, ts1.hour ?? 0, ts1.minute ?? 0);
    const utc2 = Date.UTC(ts2.year ?? 0, (ts2.month ?? 1) - 1, ts2.day ?? 1, ts2.hour ?? 0, ts2.minute ?? 0);
    if (strict === true && utc2 < utc1) {
        // Not negative number
        // utc2 - utc1 < 0  -> utc2 < utc1 ->   NO: utc1 >= utc2
        return 0;
    }
    return utc2 - utc1;
}
/**
 * Updates a {@link Timestamp} with relative data (past, current and future)
 * @param {Timestamp} timestamp The {@link Timestamp} that needs relative data updated
 * @param {Timestamp} now {@link Timestamp} that represents the current date (optional time)
 * @param {boolean=} time Optional flag to include time ('timestamp' and 'now' params should have time values)
 * @returns {Timestamp} A new {@link Timestamp}
 */
function updateRelative(timestamp, now, time = false) {
    let ts = copyTimestamp(timestamp);
    let a = getDayIdentifier(now);
    let b = getDayIdentifier(ts);
    let current = a === b;
    if (ts.hasTime && time && current) {
        a = getTimeIdentifier(now);
        b = getTimeIdentifier(ts);
        current = a === b;
    }
    ts.past = b < a;
    ts.current = current;
    ts.future = b > a;
    ts.currentWeekday = ts.weekday === now.weekday;
    return ts;
}
/**
 * Sets a Timestamp{@link Timestamp} to number of minutes past midnight (modifies hour and minutes if needed)
 * @param {Timestamp} timestamp The {@link Timestamp} to modify
 * @param {number} minutes The number of minutes to set from midnight
 * @param {Timestamp=} now Optional {@link Timestamp} representing current date and time
 * @returns {Timestamp} A new {@link Timestamp}
 */
function updateMinutes(timestamp, minutes, now = null) {
    let ts = copyTimestamp(timestamp);
    ts.hasTime = true;
    ts.hour = Math.floor(minutes / TIME_CONSTANTS.MINUTES_IN.HOUR);
    ts.minute = minutes % TIME_CONSTANTS.MINUTES_IN.HOUR;
    ts.time = getTime(ts);
    if (now) {
        ts = updateRelative(ts, now, true);
    }
    return ts;
}
/**
 * Updates the {@link Timestamp} with the weekday
 * @param {Timestamp} timestamp The {@link Timestamp} to modify
 * @returns A new Timestamp
 */
function updateWeekday(timestamp) {
    let ts = copyTimestamp(timestamp);
    ts.weekday = getWeekday(ts);
    return ts;
}
/**
 * Updates the {@link Timestamp} with the day of the year (doy)
 * @param {Timestamp} timestamp The {@link Timestamp} to modify
 * @returns A new Timestamp
 */
function updateDayOfYear(timestamp) {
    let ts = copyTimestamp(timestamp);
    ts.doy = getDayOfYear(ts) || 0;
    return ts;
}
/**
 * Updates the {@link Timestamp} with the workweek
 * @param {Timestamp} timestamp The {@link Timestamp} to modify
 * @returns A new {@link Timestamp}
 */
function updateWorkWeek(timestamp) {
    let ts = copyTimestamp(timestamp);
    ts.workweek = getWorkWeek(ts);
    return ts;
}
/**
 * Updates the passed {@link Timestamp} with disabled, if needed
 * @param {Timestamp} timestamp The {@link Timestamp} to modify
 * @param {string} [disabledBefore] In 'YYY-MM-DD' format
 * @param {string} [disabledAfter] In 'YYY-MM-DD' format
 * @param {number[]} [disabledWeekdays] An array of numbers representing weekdays [0 = Sun, ..., 6 = Sat]
 * @param {string[]|string[][]} [disabledDays] An array of days in 'YYYY-MM-DD' format. If an array with a pair of dates is in first array, then this is treated as a range.
 * @returns A new {@link Timestamp}
 */
function updateDisabled(timestamp, disabledBefore, disabledAfter, disabledWeekdays, disabledDays) {
    let ts = copyTimestamp(timestamp);
    const t = getDayIdentifier(ts);
    if (disabledBefore !== undefined) {
        const disabledDay = parsed(disabledBefore);
        if (disabledDay) {
            const before = getDayIdentifier(disabledDay);
            if (t <= before) {
                ts.disabled = true;
            }
        }
    }
    if (ts.disabled !== true && disabledAfter !== undefined) {
        const disabledDay = parsed(disabledAfter);
        if (disabledDay) {
            const after = getDayIdentifier(disabledDay);
            if (t >= after) {
                ts.disabled = true;
            }
        }
    }
    if (ts.disabled !== true && Array.isArray(disabledWeekdays) && disabledWeekdays.length > 0) {
        for (const weekday in disabledWeekdays) {
            if (disabledWeekdays[weekday] === ts.weekday) {
                ts.disabled = true;
                break;
            }
        }
    }
    if (ts.disabled !== true && Array.isArray(disabledDays) && disabledDays.length > 0) {
        for (const day in disabledDays) {
            if (Array.isArray(disabledDays[day]) &&
                disabledDays[day].length === 2 &&
                disabledDays[day][0] &&
                disabledDays[day][1]) {
                const start = parsed(disabledDays[day][0]);
                const end = parsed(disabledDays[day][1]);
                if (start && end && isBetweenDates(ts, start, end)) {
                    ts.disabled = true;
                    break;
                }
            }
            else {
                const disabledDayOrRange = disabledDays[day];
                // handle ranges with multiple days
                if (Array.isArray(disabledDayOrRange)) {
                    for (const range of disabledDayOrRange) {
                        const disabledDay = parseTimestamp(range);
                        if (disabledDay) {
                            const d = getDayIdentifier(disabledDay);
                            if (d === t) {
                                ts.disabled = true;
                                break;
                            }
                        }
                    }
                }
                else if (disabledDayOrRange) {
                    const disabledDay = parseTimestamp(disabledDayOrRange);
                    if (disabledDay) {
                        const d = getDayIdentifier(disabledDay);
                        if (d === t) {
                            ts.disabled = true;
                        }
                    }
                }
            }
        }
    }
    return ts;
}
/**
 * Updates the passed {@link Timestamp} with formatted data (time string, date string, weekday, day of year and workweek)
 * @param {Timestamp} timestamp The {@link Timestamp} to modify
 * @returns A new {@link Timestamp}
 */
function updateFormatted(timestamp) {
    let ts = copyTimestamp(timestamp);
    ts.hasTime = true;
    ts.time = getTime(ts);
    ts.date = getDate(ts);
    ts.weekday = getWeekday(ts);
    ts.doy = getDayOfYear(ts) || 0;
    ts.workweek = getWorkWeek(ts);
    return ts;
}
/**
 * Returns day of the year (doy) for the passed in {@link Timestamp}
 * @param {Timestamp} timestamp The {@link Timestamp} to use
 * @returns {number} The day of the year
 */
function getDayOfYear(timestamp) {
    if (timestamp.year === 0)
        return;
    return ((Date.UTC(timestamp.year, timestamp.month - 1, timestamp.day) -
        Date.UTC(timestamp.year, 0, 0)) /
        24 /
        60 /
        60 /
        1000);
}
/**
 * Returns workweek for the passed in {@link Timestamp}
 * @param {Timestamp} timestamp The {@link Timestamp} to use
 * @returns {number} The work week
 */
function getWorkWeek(timestamp) {
    let ts = copyTimestamp(timestamp);
    if (ts.year === 0) {
        const parsedToday = parseTimestamp(today());
        if (parsedToday) {
            ts = parsedToday;
        }
    }
    // Remove time components of date
    const weekday = new Date(Date.UTC(ts.year, ts.month - 1, ts.day));
    // Adjust the date to the correct day of the week
    const dayAdjustment = 4; // thursday is 4
    weekday.setUTCDate(weekday.getUTCDate() - ((weekday.getUTCDay() + 6) % 7) + dayAdjustment);
    // Set to nearest Thursday: current date + 4 - current day number
    // Make Sunday's day number 7
    weekday.setUTCDate(weekday.getUTCDate() + dayAdjustment - (weekday.getUTCDay() || 7));
    // Get first day of year
    var yearStart = new Date(Date.UTC(weekday.getUTCFullYear(), 0, 1));
    // Calculate full weeks to nearest Thursday
    var weekNumber = Math.ceil(((weekday.valueOf() - yearStart.valueOf()) / 86400000 + 1) / 7);
    return weekNumber;
}
/**
 * Returns weekday for the passed in {@link Timestamp}
 * @param {Timestamp} timestamp The {@link Timestamp} to use
 * @returns {number} The weekday
 */
function getWeekday(timestamp) {
    let weekday = timestamp.weekday;
    if (timestamp.hasDay) {
        const floor = Math.floor;
        const day = timestamp.day;
        const month = ((timestamp.month + 9) % MONTH_MAX) + 1;
        const century = floor(timestamp.year / 100);
        const year = (timestamp.year % 100) - (timestamp.month <= 2 ? 1 : 0);
        weekday =
            (((day +
                floor(2.6 * month - 0.2) -
                2 * century +
                year +
                floor(year / 4) +
                floor(century / 4)) %
                7) +
                7) %
                7;
    }
    return weekday ?? 0;
}
/**
 * Makes a copy of the passed in {@link Timestamp}
 * @param {Timestamp} timestamp The original {@link Timestamp}
 * @returns {Timestamp} A copy of the original {@link Timestamp}
 */
function copyTimestamp(timestamp) {
    return { ...timestamp };
}
/**
 * Used internally to convert {@link Timestamp} used with 'parsed' or 'parseDate' so the 'date' portion of the {@link Timestamp} is correct.
 * @param {Timestamp} timestamp The (raw) {@link Timestamp}
 * @returns {string} A formatted date ('YYYY-MM-DD')
 */
function getDate(timestamp) {
    let str = `${padNumber(timestamp.year, 4)}-${padNumber(timestamp.month, 2)}`;
    if (timestamp.hasDay)
        str += `-${padNumber(timestamp.day, 2)}`;
    return str;
}
/**
 * Used intenally to convert {@link Timestamp} with 'parsed' or 'parseDate' so the 'time' portion of the {@link Timestamp} is correct.
 * @param {Timestamp} timestamp The (raw) {@link Timestamp}
 * @returns {string} A formatted time ('hh:mm')
 */
function getTime(timestamp) {
    if (!timestamp.hasTime) {
        return '';
    }
    return `${padNumber(timestamp.hour, 2)}:${padNumber(timestamp.minute, 2)}`;
}
/**
 * Returns a formatted string date and time ('YYYY-YY-MM hh:mm')
 * @param {Timestamp} timestamp The {@link Timestamp}
 * @returns {string} A formatted date time ('YYYY-MM-DD HH:mm')
 */
function getDateTime(timestamp) {
    return getDate(timestamp) + ' ' + (timestamp.hasTime ? getTime(timestamp) : '00:00');
}
/**
 * An alias for {relativeDays}
 * @param {Timestamp} timestamp The {@link Timestamp} to modify
 * @param {function} [mover=nextDay] The mover function to use (ie: {nextDay} or {prevDay}).
 * @param {number} [days=1] The number of days to move.
 * @param {number[]} [allowedWeekdays=[ 0, 1, 2, 3, 4, 5, 6 ]] An array of numbers representing the weekdays. ie: [0 = Sun, ..., 6 = Sat].
 * @returns The modified {@link Timestamp}
 */
function moveRelativeDays(timestamp, mover = nextDay, days = 1, allowedWeekdays = [0, 1, 2, 3, 4, 5, 6]) {
    const ts = copyTimestamp(timestamp);
    return relativeDays(ts, mover, days, allowedWeekdays);
}
/**
 * Moves the {@link Timestamp} the number of relative days
 * @param {Timestamp} timestamp The {@link Timestamp} to modify
 * @param {function} [mover=nextDay] The mover function to use (ie: {nextDay} or {prevDay}).
 * @param {number} [days=1] The number of days to move.
 * @param {number[]} [allowedWeekdays=[ 0, 1, 2, 3, 4, 5, 6 ]] An array of numbers representing the weekdays. ie: [0 = Sun, ..., 6 = Sat].
 * @returns A new {@link Timestamp}
 */
function relativeDays(timestamp, mover = nextDay, days = 1, allowedWeekdays = [0, 1, 2, 3, 4, 5, 6]) {
    let ts = copyTimestamp(timestamp);
    if (!allowedWeekdays.includes(Number(ts.weekday)) && ts.weekday === 0 && mover === nextDay) {
        ++days;
    }
    while (--days >= 0) {
        ts = mover(ts);
        if (allowedWeekdays.length < 7 && !allowedWeekdays.includes(Number(ts.weekday))) {
            ++days;
        }
    }
    return ts;
}
/**
 * Finds the specified weekday (forward or back) based on the {@link Timestamp}
 * @param {Timestamp} timestamp The {@link Timestamp} to modify
 * @param {number} weekday The weekday number (Sun = 0, ..., Sat = 6)
 * @param {function} [mover=nextDay] The function to use ({prevDay} or {nextDay}).
 * @param {number} [maxDays=6] The number of days to look forward or back.
 * @returns A new {@link Timestamp}
 */
function findWeekday(timestamp, weekday, mover = nextDay, maxDays = 6) {
    let ts = copyTimestamp(timestamp);
    while (ts.weekday !== weekday && --maxDays >= 0)
        ts = mover(ts);
    return ts;
}
/**
 * Creates an array of {@link Timestamp}s based on start and end params
 * @param {Timestamp} start The starting {@link Timestamp}
 * @param {Timestamp} end The ending {@link Timestamp}
 * @param {Timestamp} now The relative day
 * @param {number[]} weekdays An array of numbers (representing days of the week) that are 0 (=Sunday) to 6 (=Saturday)
 * @param {string} [disabledBefore] Days before this date are disabled (YYYY-MM-DD)
 * @param {string} [disabledAfter] Days after this date are disabled (YYYY-MM-DD)
 * @param {number[]} [disabledWeekdays] An array representing weekdays that are disabled [0 = Sun, ..., 6 = Sat]
 * @param {string[]} [disabledDays] An array of days in 'YYYY-MM-DD' format. If an array with a pair of dates is in first array, then this is treated as a range.
 * @param {number} [max=42] Max days to do
 * @param {number} [min=0]  Min days to do
 * @returns {Timestamp[]} The requested array of {@link Timestamp}s
 */
function createDayList(start, end, now, weekdays = [0, 1, 2, 3, 4, 5, 6], disabledBefore = undefined, disabledAfter = undefined, disabledWeekdays = [], disabledDays = [], max = 42, min = 0) {
    const begin = getDayIdentifier(start);
    const stop = getDayIdentifier(end);
    const days = [];
    let current = copyTimestamp(start);
    let currentIdentifier = 0;
    let stopped = currentIdentifier === stop;
    if (stop < begin) {
        return days;
    }
    while ((!stopped || days.length < min) && days.length < max) {
        currentIdentifier = getDayIdentifier(current);
        stopped = stopped || (currentIdentifier > stop && days.length >= min);
        if (stopped) {
            break;
        }
        if (!weekdays.includes(Number(current.weekday))) {
            current = relativeDays(current, nextDay);
            continue;
        }
        let day = copyTimestamp(current);
        day = updateFormatted(day);
        day = updateRelative(day, now);
        day = updateDisabled(day, disabledBefore, disabledAfter, disabledWeekdays, disabledDays);
        days.push(day);
        current = relativeDays(current, nextDay);
    }
    return days;
}
/**
 * Creates an array of interval {@link Timestamp}s based on params
 * @param {Timestamp} timestamp The starting {@link Timestamp}
 * @param {number} first The starting interval time
 * @param {number} minutes How many minutes between intervals (ie: 60, 30, 15 would be common ones)
 * @param {number} count The number of intervals needed
 * @param {Timestamp} now A relative {@link Timestamp} with time
 * @returns {Timestamp[]} The requested array of interval {@link Timestamp}s
 */
function createIntervalList(timestamp, first, minutes, count, now) {
    const intervals = [];
    for (let i = 0; i < count; ++i) {
        const mins = (first + i) * minutes;
        const ts = copyTimestamp(timestamp);
        intervals.push(updateMinutes(ts, mins, now));
    }
    return intervals;
}
/**
 * @callback getOptions
 * @param {Timestamp} timestamp A {@link Timestamp} object
 * @param {boolean} short True if using short options
 * @returns {Object} An Intl object representing optioons to be used
 */
/**
 * @callback formatter
 * @param {Timestamp} timestamp The {@link Timestamp} being used
 * @param {boolean} short If short format is being requested
 * @returns {string} The localized string of the formatted {@link Timestamp}
 */
/**
 * Returns a function that uses Intl.DateTimeFormat formatting
 * @param {string} locale The locale to use (ie: en-US)
 * @param {getOptions} cb The function to call for options. This function should return an Intl formatted object. The function is passed (timestamp, short).
 * @returns {formatter} The function has params (timestamp, short). The short is to use the short options.
 */
function createNativeLocaleFormatter(locale, cb) {
    const emptyFormatter = () => '';
    /* istanbul ignore next */
    if (typeof Intl === 'undefined' || typeof Intl.DateTimeFormat === 'undefined') {
        return emptyFormatter;
    }
    return (timestamp, short) => {
        try {
            const intlFormatter = new Intl.DateTimeFormat(locale || undefined, cb(timestamp, short));
            return intlFormatter.format(makeDateTime(timestamp));
        }
        catch (e) /* istanbul ignore next */ {
            console.error(`Intl.DateTimeFormat: ${e.message} -> ${getDateTime(timestamp)}`);
            return '';
        }
    };
}
/**
 * Makes a JavaScript Date from the passed {@link Timestamp}
 * @param {Timestamp} timestamp The {@link Timestamp} to use
 * @param {boolean} utc True to get Date object using UTC
 * @returns {Date} A JavaScript Date
 */
function makeDate(timestamp, utc = true) {
    if (utc)
        return new Date(Date.UTC(timestamp.year, timestamp.month - 1, timestamp.day, 0, 0));
    return new Date(timestamp.year, timestamp.month - 1, timestamp.day, 0, 0);
}
/**
 * Makes a JavaScript Date from the passed {@link Timestamp} (with time)
 * @param {Timestamp} timestamp The {@link Timestamp} to use
 * @param {boolean} utc True to get Date object using UTC
 * @returns {Date} A JavaScript Date
 */
function makeDateTime(timestamp, utc = true) {
    if (utc)
        return new Date(Date.UTC(timestamp.year, timestamp.month - 1, timestamp.day, timestamp.hour, timestamp.minute));
    return new Date(timestamp.year, timestamp.month - 1, timestamp.day, timestamp.hour, timestamp.minute);
}
/**
 * Validates if the input is a finite number.
 *
 * @param input - The value to be validated. Can be a string or a number.
 * @returns A boolean indicating whether the input is a finite number.
 *          Returns true if the input is a finite number, false otherwise.
 */
function validateNumber(input) {
    return isFinite(Number(input));
}
/**
 * Given an array of {@link Timestamp}s, finds the max date (and possible time)
 * @param {Timestamp[]} timestamps This is an array of {@link Timestamp}s
 * @param {boolean=} useTime Default false; if true, uses time in the comparison as well
 * @returns The {@link Timestamp} with the highest date (and possibly time) value
 */
function maxTimestamp(timestamps, useTime = false) {
    const func = useTime === true ? getDayTimeIdentifier : getDayIdentifier;
    return timestamps.reduce((prev, cur) => {
        return Math.max(func(prev), func(cur)) === func(prev) ? prev : cur;
    });
}
/**
 * Given an array of {@link Timestamp}s, finds the min date (and possible time)
 * @param {Timestamp[]} timestamps This is an array of {@link Timestamp}s
 * @param {boolean=} useTime Default false; if true, uses time in the comparison as well
 * @returns The {@link Timestamp} with the lowest date (and possibly time) value
 */
function minTimestamp(timestamps, useTime = false) {
    const func = useTime === true ? getDayTimeIdentifier : getDayIdentifier;
    return timestamps.reduce((prev, cur) => {
        return Math.min(func(prev), func(cur)) === func(prev) ? prev : cur;
    });
}
/**
 * Determines if the passed {@link Timestamp} is between (or equal) to two {@link Timestamp}s (range)
 * @param {Timestamp} timestamp The {@link Timestamp} for testing
 * @param {Timestamp} startTimestamp The starting {@link Timestamp}
 * @param {Timestamp} endTimestamp The ending {@link Timestamp}
 * @param {boolean=} useTime If true, use time from the {@link Timestamp}s
 * @returns {boolean} True if {@link Timestamp} is between (or equal) to two {@link Timestamp}s (range)
 */
function isBetweenDates(timestamp, startTimestamp, endTimestamp, useTime = false) {
    const cd = getDayIdentifier(timestamp) + (useTime === true ? getTimeIdentifier(timestamp) : 0);
    const sd = getDayIdentifier(startTimestamp) + (useTime === true ? getTimeIdentifier(startTimestamp) : 0);
    const ed = getDayIdentifier(endTimestamp) + (useTime === true ? getTimeIdentifier(endTimestamp) : 0);
    return cd >= sd && cd <= ed;
}
/**
 * Determine if two ranges of {@link Timestamp}s overlap each other
 * @param {Timestamp} startTimestamp The starting {@link Timestamp} of first range
 * @param {Timestamp} endTimestamp The endinging {@link Timestamp} of first range
 * @param {Timestamp} firstTimestamp The starting {@link Timestamp} of second range
 * @param {Timestamp} lastTimestamp The ending {@link Timestamp} of second range
 * @returns {boolean} True if the two ranges overlap each other
 */
function isOverlappingDates(startTimestamp, endTimestamp, firstTimestamp, lastTimestamp) {
    const start = getDayIdentifier(startTimestamp);
    const end = getDayIdentifier(endTimestamp);
    const first = getDayIdentifier(firstTimestamp);
    const last = getDayIdentifier(lastTimestamp);
    return ((start >= first && start <= last) || // overlap left
        (end >= first && end <= last) || // overlap right
        (first >= start && end >= last) // surrounding
    );
}
/**
 * Add or decrements years, months, days, hours or minutes to a timestamp
 * @param {Timestamp} timestamp The {@link Timestamp} object
 * @param {Object} options configuration data
 * @param {number=} options.year If positive, adds years. If negative, removes years.
 * @param {number=} options.month If positive, adds months. If negative, removes month.
 * @param {number=} options.day If positive, adds days. If negative, removes days.
 * @param {number=} options.hour If positive, adds hours. If negative, removes hours.
 * @param {number=} options.minute If positive, adds minutes. If negative, removes minutes.
 * @returns {Timestamp} A modified copy of the passed in {@link Timestamp}
 */
function addToDate(timestamp, options) {
    const ts = copyTimestamp(timestamp);
    if (options.year)
        ts.year += options.year;
    if (options.month)
        ts.month += options.month;
    if (options.day)
        ts.day += options.day;
    if (options.hour)
        ts.hour += options.hour;
    if (options.minute)
        ts.minute += options.minute;
    return updateFormatted(normalizeTimestamp(ts));
}
/**
 * Normalizes a timestamp object by creating a JavaScript Date object and extracting standardized values.
 * This function ensures that the timestamp values are consistent and correctly represent a valid date and time.
 *
 * @param {Object} ts - The timestamp object to normalize.
 * @param {number} ts.year - The year of the timestamp.
 * @param {number} ts.month - The month of the timestamp (1-12).
 * @param {number} ts.day - The day of the month.
 * @param {number} ts.hour - The hour of the day (0-23).
 * @param {number} ts.minute - The minute of the hour (0-59).
 * @returns {Object} A new object with normalized timestamp values.
 *                   The returned object includes all properties from the input object,
 *                   with year, month, day, hour, and minute properties updated to normalized values.
 */
function normalizeTimestamp(ts) {
    const date = new Date(ts.year, ts.month - 1, ts.day, ts.hour, ts.minute);
    return {
        ...ts,
        year: date.getFullYear(),
        month: date.getMonth() + 1,
        day: date.getDate(),
        hour: date.getHours(),
        minute: date.getMinutes(),
    };
}
/**
 * Returns number of days between two {@link Timestamp}s
 * @param {Timestamp} ts1 The first {@link Timestamp}
 * @param {Timestamp} ts2 The second {@link Timestamp}
 * @returns Number of days
 */
function daysBetween(ts1, ts2) {
    const diff = diffTimestamp(ts1, ts2, true);
    return Math.floor(diff / TIME_CONSTANTS.MILLISECONDS_IN.DAY);
}
/**
 * Returns number of weeks between two {@link Timestamp}s
 * @param {Timestamp} ts1 The first {@link Timestamp}
 * @param {Timestamp} ts2 The second {@link Timestamp}
 */
function weeksBetween(ts1, ts2) {
    let t1 = copyTimestamp(ts1);
    let t2 = copyTimestamp(ts2);
    t1 = findWeekday(t1, 0);
    t2 = findWeekday(t2, 6);
    return Math.ceil(daysBetween(t1, t2) / TIME_CONSTANTS.DAYS_IN.WEEK);
}
// Known dates
const weekdayDateMap = {
    Sun: new Date('2020-01-05T00:00:00.000Z'),
    Mon: new Date('2020-01-06T00:00:00.000Z'),
    Tue: new Date('2020-01-07T00:00:00.000Z'),
    Wed: new Date('2020-01-08T00:00:00.000Z'),
    Thu: new Date('2020-01-09T00:00:00.000Z'),
    Fri: new Date('2020-01-10T00:00:00.000Z'),
    Sat: new Date('2020-01-11T00:00:00.000Z'),
};
/**
 * Returns a function that uses Intl.DateTimeFormat to format weekdays.
 *
 * @function getWeekdayFormatter
 * @returns {function} A function that formats weekdays.
 *
 * @example
 * const formatWeekday = getWeekdayFormatter();
 * console.log(formatWeekday('Mon', 'long', 'en-US')); // "Monday"
 * console.log(formatWeekday('Mon', 'short', 'fr-FR')); // "lun."
 *
 * @param {string} weekday - The abbreviation of the weekday (e.g., 'Mon', 'Tue', 'Wed', etc.).
 * @param {string} [type='long'] - The type of formatting to use ('narrow', 'short', or 'long').
 * @param {string} [locale=''] - The locale to use for formatting.
 *
 * @returns {string} The formatted weekday.
 */
function getWeekdayFormatter() {
    const emptyFormatter = () => '';
    const options = {
        long: { timeZone: 'UTC', weekday: 'long' },
        short: { timeZone: 'UTC', weekday: 'short' },
        narrow: { timeZone: 'UTC', weekday: 'narrow' },
    };
    /* istanbul ignore next */
    if (typeof Intl === 'undefined' || typeof Intl.DateTimeFormat === 'undefined') {
        return emptyFormatter;
    }
    /**
     * Formats a given weekday into a localized string based on the specified type and locale.
     *
     * @param {number} weekday - The day of the week (0 for Sunday, 1 for Monday, etc.).
     * @param {string} type - The format type (e.g., 'narrow', 'short', 'long') to use for formatting.
     * @param {string} [locale] - The locale string (e.g., 'en-US') to use for formatting. Defaults to the user's locale if not provided.
     * @returns {string} The formatted weekday string.
     */
    function weekdayFormatter(weekday, type, locale) {
        try {
            const intlFormatter = new Intl.DateTimeFormat(locale || undefined, 
            /// @ts-expect-error ignore for now
            options[type] || options['long']);
            return intlFormatter.format(weekdayDateMap[weekday]);
        }
        catch (e) /* istanbul ignore next */ {
            if (e instanceof Error) {
                console.error(`Intl.DateTimeFormat: ${e.message} -> day of week: ${weekday}`);
            }
            return '';
        }
    }
    return weekdayFormatter;
}
/**
 * Retrieves an array of localized weekday names.
 *
 * @param {string} type - The format type for the weekday names. Can be 'narrow', 'short', or 'long'.
 * @param {string} [locale] - The locale to use for formatting. If not provided, the default locale is used.
 * @returns {string[]} An array of localized weekday names in the specified format.
 */
function getWeekdayNames(type, locale) {
    const shortWeekdays = Object.keys(weekdayDateMap);
    const weekdayFormatter = getWeekdayFormatter();
    return shortWeekdays.map((weekday) => String(weekdayFormatter(weekday, type, locale)));
}
/**
 * Creates and returns a function for formatting month names based on locale and format type.
 *
 * @returns {Function} A function that formats month names.
 *   The returned function accepts the following parameters:
 *   @param {number} month - The month to format (0-11, where 0 is January).
 *   @param {string} [type='long'] - The format type: 'narrow', 'short', or 'long'.
 *   @param {string} [locale] - The locale to use for formatting. If not provided, the default locale is used.
 *   @returns {string} The formatted month name.
 *
 * @throws {Error} If Intl or Intl.DateTimeFormat is not supported in the environment.
 */
function getMonthFormatter() {
    const emptyFormatter = () => '';
    const options = {
        long: { timeZone: 'UTC', month: 'long' },
        short: { timeZone: 'UTC', month: 'short' },
        narrow: { timeZone: 'UTC', month: 'narrow' },
    };
    /* istanbul ignore next */
    if (typeof Intl === 'undefined' || typeof Intl.DateTimeFormat === 'undefined') {
        return emptyFormatter;
    }
    /**
     * Formats a given month into a string based on the specified type and locale.
     *
     * @param {number} month - The month to format (0 for January, 11 for December).
     * @param {string} type - The format type (e.g., 'narrow', 'long', 'short', etc.).
     * @param {string} [locale] - The locale to use for formatting (defaults to the system locale if not provided).
     * @returns {string} The formatted month string.
     */
    function monthFormatter(month, type, locale) {
        try {
            const intlFormatter = new Intl.DateTimeFormat(locale || undefined, 
            /// @ts-expect-error ignore for now
            options[type] || options['long']);
            const date = new Date();
            date.setDate(1);
            date.setMonth(month);
            return intlFormatter.format(date);
        }
        catch (e) /* istanbul ignore next */ {
            if (e instanceof Error) {
                console.error(`Intl.DateTimeFormat: ${e.message} -> month: ${month}`);
            }
            return '';
        }
    }
    return monthFormatter;
}
/**
 * Retrieves an array of localized month names.
 *
 * @param {string} type - The format type for the month names. Can be 'narrow', 'short', or 'long'.
 * @param {string} [locale] - The locale to use for formatting. If not provided, the default locale is used.
 * @returns {string[]} An array of localized month names in the specified format.
 */
function getMonthNames(type, locale) {
    const monthFormatter = getMonthFormatter();
    return [...Array(12).keys()].map((month) => monthFormatter(month, type, locale));
}

function convertToUnit(input, unit = 'px') {
    if (!input) {
        return undefined;
    }
    else if (isNaN(input)) {
        return String(input);
    }
    else if (input === 'auto') {
        return input;
    }
    else {
        return `${Number(input)}${unit}`;
    }
}
function indexOf(array, cb) {
    for (let i = 0; i < array.length; i++) {
        if (cb(array[i], i) === true) {
            return i;
        }
    }
    return -1;
}
function minCharWidth(str, count) {
    if (count === 0)
        return str;
    return str.slice(0, count);
}

/**
 * Converts a kebab-case string to camelCase.
 * @param {string} str - The kebab-case string.
 * @returns {string} - The camelCase formatted string.
 */
const toCamelCase = (str) => str.replace(/(-\w)/g, (m) => m[1].toUpperCase());
/**
 * Generates mouse event handlers based on event listeners.
 * @param {Function} emit - Vue's emit function.
 * @param {Ref<Listeners>} listeners - Vue ref containing event listeners.
 * @param {Record<string, MouseEventOptions>} events - Object defining mouse events and their options.
 * @param {GetEventFunction} getEvent - Function to transform event objects.
 * @returns {Record<string, Function | Function[]>} - The mapped mouse event handlers.
 */
function getMouseEventHandlers(emit, listeners, events, getEvent) {
    const on = {};
    for (const eventName in events) {
        const eventOptions = events[eventName];
        const eventKey = toCamelCase('on-' + eventName);
        if (!listeners.value) {
            console.warn('$listeners has not been set up');
            return {};
        }
        if (listeners.value[eventKey] === undefined)
            continue;
        const key = 'on' + eventOptions.event.charAt(0).toUpperCase() + eventOptions.event.slice(1);
        const handler = (event) => {
            if (eventOptions.button === undefined ||
                ('buttons' in event && event.buttons > 0 && event.button === eventOptions.button)) {
                if (eventOptions.prevent) {
                    event.preventDefault();
                }
                if (eventOptions.stop) {
                    event.stopPropagation();
                }
                emit(eventName, getEvent(event, eventName));
            }
            return eventOptions.result;
        };
        if (key in on) {
            if (Array.isArray(on[key])) {
                on[key].push(handler);
            }
            else {
                on[key] = [on[key], handler];
            }
        }
        else {
            on[key] = handler;
        }
    }
    return on;
}
/**
 * Returns default mouse event handlers based on a suffix.
 * @param {Function} emit - Vue's emit function.
 * @param {Ref<Listeners>} listeners - Vue ref containing event listeners.
 * @param {string} suffix - Event suffix (e.g., '-day' for 'click-day').
 * @param {GetEventFunction} getEvent - Function to transform event objects.
 * @returns {Record<string, Function | Function[]>} - The mapped event handlers.
 */
function getDefaultMouseEventHandlers(emit, listeners, suffix, getEvent) {
    return getMouseEventHandlers(emit, listeners, getMouseEventName(suffix), getEvent);
}
/**
 * Generates event names for different mouse interactions.
 * @param {string} suffix - Event suffix (e.g., '-day' for 'click-day').
 * @returns {Record<string, MouseEventOptions>} - The event name map.
 */
function getMouseEventName(suffix) {
    return {
        ['click' + suffix]: { event: 'click' },
        ['contextmenu' + suffix]: { event: 'contextmenu', prevent: true, result: false },
        ['mousedown' + suffix]: { event: 'mousedown' },
        ['mousemove' + suffix]: { event: 'mousemove' },
        ['mouseup' + suffix]: { event: 'mouseup' },
        ['mouseenter' + suffix]: { event: 'mouseenter' },
        ['mouseleave' + suffix]: { event: 'mouseleave' },
        ['touchstart' + suffix]: { event: 'touchstart' },
        ['touchmove' + suffix]: { event: 'touchmove' },
        ['touchend' + suffix]: { event: 'touchend' },
    };
}
/**
 * Returns an array of raw event names based on a suffix.
 * @param {string} suffix - Event suffix.
 * @returns {string[]} - The raw event names.
 */
function getRawMouseEvents(suffix) {
    return Object.keys(getMouseEventName(suffix));
}
/**
 * Default function export that provides mouse event handling utilities.
 * @param {Function} emit - Vue's emit function.
 * @param {Ref<Listeners>} listeners - Vue ref containing event listeners.
 * @returns {Object} - Functions for managing mouse event handlers.
 */
function useMouseEvents(emit, listeners) {
    return {
        getMouseEventHandlers: (events, getEvent) => getMouseEventHandlers(emit, listeners, events, getEvent),
        getDefaultMouseEventHandlers: (suffix, getEvent) => getDefaultMouseEventHandlers(emit, listeners, suffix, getEvent),
        getMouseEventName,
        getRawMouseEvents,
    };
}

const ResizeObserverDirective = {
    mounted(el, binding) {
        // Ensure a callback is provided.
        if (typeof binding.value !== 'function')
            return;
        const callback = binding.value;
        const data = {
            callback,
            size: { width: 0, height: 0 },
            observer: new ResizeObserver((entries) => {
                const rect = entries[0].contentRect;
                // Check if the dimensions have changed.
                if (rect.width !== data.size.width || rect.height !== data.size.height) {
                    data.size.width = rect.width;
                    data.size.height = rect.height;
                    // Clear any pending debounce timeout.
                    if (data.debounceTimeout) {
                        clearTimeout(data.debounceTimeout);
                    }
                    // Set up a new debounce timeout.
                    data.debounceTimeout = setTimeout(() => {
                        data.callback(data.size);
                        data.debounceTimeout = undefined;
                    }, 100); // Adjust the debounce delay (in ms) as needed.
                }
            }),
        };
        // Start observing the element.
        data.observer.observe(el);
        // Store the data on the element.
        el.__onResizeObserver = data;
    },
    beforeUnmount(el) {
        if (!el.__onResizeObserver)
            return;
        const { observer, debounceTimeout } = el.__onResizeObserver;
        // Clear any pending debounce.
        if (debounceTimeout) {
            clearTimeout(debounceTimeout);
        }
        // Stop observing the element.
        observer.unobserve(el);
        // Clean up the property.
        delete el.__onResizeObserver;
    },
};

/**
 * The main QCalendar wrapper
 * All others are a child to this one
 */
// Vue
function useCalendar(props, renderFunc, { scrollArea, pane }) {
    if (!renderFunc) {
        const msg = '[error: renderCalendar] no renderFunc has been supplied to useCalendar';
        console.error(msg);
        throw new Error(msg);
    }
    const size = vue.reactive({ width: 0, height: 0 }), rootRef = vue.ref(null);
    function __onResize({ width, height }) {
        size.width = width;
        size.height = height;
    }
    const scrollWidth = vue.computed(() => {
        return props.noScroll !== true
            ? scrollArea.value && pane.value && size.height // force recalc with height change
                ? scrollArea.value.offsetWidth - pane.value.offsetWidth
                : 0
            : 0;
    });
    function __initCalendar() {
        // required
    }
    function __renderCalendar() {
        const data = {
            ref: rootRef,
            role: 'complementary',
            lang: props.locale,
            class: `q-calendar ${props.dark ? 'q-calendar--dark' : ''} ${props.bordered ? 'q-calendar__bordered' : ''}`,
        };
        return vue.withDirectives(vue.h('div', { ...data }, [renderFunc()]), [[ResizeObserverDirective, __onResize]]);
    }
    return {
        rootRef,
        scrollWidth,
        __initCalendar,
        __renderCalendar,
    };
}

const isValidFocusType = (v) => v.every((type) => ['day', 'date', 'weekday', 'interval', 'time', 'resource', 'task'].includes(type));
// Define prop types with validators
const useCommonProps = {
    modelValue: {
        type: String,
        default: today(),
        validator: (v) => v === '' || validateTimestamp(v),
    },
    weekdays: {
        type: Array,
        default: () => [0, 1, 2, 3, 4, 5, 6],
    },
    dateType: {
        type: String,
        default: 'round',
        validator: (v) => ['round', 'rounded', 'square'].includes(v),
    },
    weekdayAlign: {
        type: String,
        default: 'center',
        validator: (v) => ['left', 'center', 'right'].includes(v),
    },
    dateAlign: {
        type: String,
        default: 'center',
        validator: (v) => ['left', 'center', 'right'].includes(v),
    },
    bordered: Boolean,
    dark: Boolean,
    noAria: Boolean,
    noActiveDate: Boolean,
    noHeader: Boolean,
    noScroll: Boolean,
    shortWeekdayLabel: Boolean,
    noDefaultHeaderText: Boolean,
    noDefaultHeaderBtn: Boolean,
    minWeekdayLabel: {
        type: [Number, String],
        default: 1,
    },
    weekdayBreakpoints: {
        type: Array,
        default: () => [75, 35],
        validator: (v) => v.length === 2,
    },
    locale: {
        type: String,
        default: 'en-US',
    },
    animated: Boolean,
    transitionPrev: {
        type: String,
        default: 'slide-right',
    },
    transitionNext: {
        type: String,
        default: 'slide-left',
    },
    disabledDays: Array,
    disabledBefore: String,
    disabledAfter: String,
    disabledWeekdays: {
        type: Array,
        default: () => [],
    },
    dragEnterFunc: Function,
    dragOverFunc: Function,
    dragLeaveFunc: Function,
    dropFunc: Function,
    selectedDates: {
        type: [Array, Set],
        default: () => [],
    },
    selectedStartEndDates: {
        type: Array,
        default: () => [],
    },
    hoverable: Boolean,
    focusable: Boolean,
    focusType: {
        type: Array,
        default: () => [
            'date',
        ],
        validator: isValidFocusType,
    },
};
function useCommon(props, { startDate, endDate, times, }) {
    const parsedStart = vue.computed(() => parseTimestamp(startDate.value));
    const parsedEnd = vue.computed(() => {
        if (endDate.value === '0000-00-00') {
            return getEndOfWeek(parsedStart.value, props.weekdays, times.today);
        }
        return parseTimestamp(endDate.value) || parsedStart.value;
    });
    const dayFormatter = vue.computed(() => createNativeLocaleFormatter(props.locale, () => ({
        timeZone: 'UTC',
        day: 'numeric',
    })));
    const weekdayFormatter = vue.computed(() => createNativeLocaleFormatter(props.locale, (_tms, short) => ({
        timeZone: 'UTC',
        weekday: short ? 'short' : 'long',
    })));
    const ariaDateFormatter = vue.computed(() => createNativeLocaleFormatter(props.locale, () => ({
        timeZone: 'UTC',
        dateStyle: 'full',
    })));
    function arrayHasDate(arr, timestamp) {
        return arr && arr.length > 0 && arr.includes(timestamp.date);
    }
    function checkDays(arr, timestamp) {
        const days = { firstDay: false, betweenDays: false, lastDay: false };
        if (arr.length === 2) {
            const current = getDayIdentifier(timestamp);
            const first = getDayIdentifier(parsed(arr[0]));
            const last = getDayIdentifier(parsed(arr[1]));
            days.firstDay = first === current;
            days.lastDay = last === current;
            days.betweenDays = first < current && last > current;
        }
        return days;
    }
    function getRelativeClasses(timestamp, outside = false, selectedDays = [], startEndDays = [], hover = false) {
        const isSelected = arrayHasDate(selectedDays, timestamp);
        const { firstDay, lastDay, betweenDays } = checkDays(startEndDays, timestamp);
        return {
            'q-past-day': !firstDay && !betweenDays && !lastDay && !isSelected && !outside && !!timestamp.past,
            'q-future-day': !firstDay && !betweenDays && !lastDay && !isSelected && !outside && !!timestamp.future,
            'q-outside': outside,
            'q-current-day': !!timestamp.current,
            'q-selected': isSelected,
            'q-range-first': firstDay,
            'q-range': betweenDays,
            'q-range-last': lastDay,
            'q-range-hover': hover && (firstDay || lastDay || betweenDays),
            'q-disabled-day disabled': timestamp.disabled === true,
        };
    }
    /**
     * Calculates the start of the week for the given timestamp.
     *
     * @param timestamp - The timestamp to calculate the start of the week for.
     * @returns The timestamp representing the start of the week.
     */
    function startOfWeek(timestamp) {
        return getStartOfWeek(timestamp, props.weekdays, times.today);
    }
    /**
     * Calculates the end of the week for the given timestamp.
     *
     * @param timestamp - The timestamp to calculate the end of the week for.
     * @returns The timestamp representing the end of the week.
     */
    function endOfWeek(timestamp) {
        return getEndOfWeek(timestamp, props.weekdays, times.today);
    }
    /**
     * Provides the default style for a day in the calendar.
     *
     * This function returns `undefined`, which means that no additional styles will be applied to the day.
     */
    function dayStyleDefault() {
        return {};
    }
    return {
        parsedStart,
        parsedEnd,
        dayFormatter,
        weekdayFormatter,
        ariaDateFormatter,
        arrayHasDate,
        checkDays,
        getRelativeClasses,
        startOfWeek,
        endOfWeek,
        dayStyleDefault,
    };
}

const useTaskProps = {
    modelValue: {
        type: String,
        default: today(),
        validator: (v) => v === '' || validateTimestamp(v),
    },
    modelTasks: {
        type: Array,
        default: () => [],
    },
    modelTitle: {
        type: Array,
        default: () => [],
    },
    modelFooter: {
        type: Array,
        default: () => [],
    },
    taskKey: {
        type: [String, Number],
        default: 'id',
    },
    weekdays: {
        type: Array,
        default: () => [0, 1, 2, 3, 4, 5, 6],
    },
    dateType: {
        type: String,
        default: 'round',
        validator: (v) => ['round', 'rounded', 'square'].includes(v),
    },
    dateHeader: {
        type: String,
        default: 'stacked',
        validator: (v) => ['stacked', 'inline', 'inverted'].includes(v),
    },
    weekdayAlign: {
        type: String,
        default: 'center',
        validator: (v) => ['left', 'center', 'right'].includes(v),
    },
    dateAlign: {
        type: String,
        default: 'center',
        validator: (v) => ['left', 'center', 'right'].includes(v),
    },
    dayHeight: {
        type: [Number, String],
        default: 0,
        validator: validateNumber,
    },
    dayMinHeight: {
        type: [Number, String],
        default: 40,
        validator: validateNumber,
    },
    weekdayStyle: {
        type: Function,
        default: null,
    },
    weekdayClass: {
        type: Function,
        default: null,
    },
    dayStyle: {
        type: Function,
        default: null,
    },
    dayClass: {
        type: Function,
        default: null,
    },
    footerDayClass: {
        type: Function,
        default: null,
    },
    view: {
        type: String,
        validator: (v) => ['day', 'week', 'month'].includes(v),
    },
    viewCount: {
        type: Number,
        default: 1,
        validator: (v) => validateNumber(v) && v > 0,
    },
    taskWidth: {
        type: Number,
        default: 200,
        validator: (v) => validateNumber(v) && v > 0,
    },
};
function useTask(props, emit, { times, }) {
    const parsedStartDate = vue.computed(() => {
        if (!props.modelValue) {
            throw new Error(`QCalendarTask: no modelValue provided`);
        }
        if (props.view === 'day') {
            return parseTimestamp(props.modelValue);
        }
        else if (props.view === 'week') {
            return getStartOfWeek(parseTimestamp(props.modelValue), props.weekdays, times.today);
        }
        else if (props.view === 'month') {
            return getStartOfMonth(parseTimestamp(props.modelValue));
        }
        else {
            throw new Error(`QCalendarTask: unknown 'view' type (${props.view})`);
        }
    });
    const parsedEndDate = vue.computed(() => {
        if (!props.modelValue) {
            throw new Error(`QCalendarTask: no modelValue provided`);
        }
        if (props.view === 'day') {
            if (props.viewCount === 1) {
                return parsedStartDate.value;
            }
            let end = copyTimestamp(parsedStartDate.value);
            end = addToDate(end, { day: props.viewCount - 1 });
            return end;
        }
        else if (props.view === 'week') {
            if (props.viewCount === 1) {
                return getEndOfWeek(parseTimestamp(props.modelValue), props.weekdays, times.today);
            }
            else {
                let end = copyTimestamp(parsedStartDate.value);
                end = addToDate(end, { day: (props.viewCount - 1) * TIME_CONSTANTS.DAYS_IN.WEEK });
                return getEndOfWeek(end, props.weekdays, times.today);
            }
        }
        else if (props.view === 'month') {
            if (props.viewCount === 1) {
                return getEndOfMonth(parseTimestamp(props.modelValue));
            }
            else {
                let end = copyTimestamp(parsedStartDate.value);
                end = addToDate(end, { month: props.viewCount });
                return getEndOfMonth(end);
            }
        }
        else {
            throw new Error(`QCalendarTask: unknown 'view' type (${props.view})`);
        }
    });
    const days = vue.computed(() => {
        return createDayList(parsedStartDate.value, parsedEndDate.value, times.today, props.weekdays, props.disabledBefore, props.disabledAfter, props.disabledWeekdays || [], props.disabledDays || [], Number.MAX_SAFE_INTEGER);
    });
    return {
        days,
        parsedStartDate,
        parsedEndDate,
    };
}

/**
 * export of useTimesProps
 */
const useTimesProps = {
    now: {
        type: String,
        validator: (v) => v === '' || validateTimestamp(v),
        default: '',
    },
};
/**
 * Reactive timestamps & computed properties for time tracking
 */
function useTimes(props) {
    /**
     * 'times' is a reactive object containing 'now' and 'today'
     */
    const times = vue.reactive({
        now: parseTimestamp('0000-00-00 00:00'),
        today: parseTimestamp('0000-00-00'),
    });
    /**
     * Parsed current timestamp
     */
    const parsedNow = vue.computed(() => (props.now ? parseTimestamp(props.now) : getNow()));
    /**
     * Watch for changes in parsedNow
     */
    vue.watch(parsedNow, () => updateCurrent());
    /**
     * Sets 'times.now' (relative) to 'times.today' (relative)
     */
    function setCurrent() {
        if (times.now && times.today) {
            times.now.current = times.today.current = true;
            times.now.past = times.today.past = false;
            times.now.future = times.today.future = false;
        }
    }
    /**
     * Updates current timestamps
     */
    function updateCurrent() {
        const now = parsedNow.value || getNow();
        updateDay(now, times.now);
        updateTime(now, times.now);
        updateDay(now, times.today);
    }
    /**
     * Get the current date as a Timestamp
     */
    function getNow() {
        return parseDate(new Date());
    }
    /**
     * Update date info of target timestamp
     */
    function updateDay(now, target) {
        if (now.date !== target.date) {
            target.year = now.year;
            target.month = now.month;
            target.day = now.day;
            target.weekday = now.weekday;
            target.date = now.date;
        }
    }
    /**
     * Update time info of target timestamp
     */
    function updateTime(now, target) {
        if (now.time !== target.time) {
            target.hour = now.hour;
            target.minute = now.minute;
            target.time = now.time;
        }
    }
    return {
        times,
        parsedNow,
        setCurrent,
        updateCurrent,
        getNow,
        updateDay,
        updateTime,
    };
}

function useRenderValues(props, { parsedView, parsedValue, times }) {
    /**
     * Computes the start, end, and maxDays based on the given view
     */
    const renderValues = vue.computed(() => {
        const around = parsedValue.value;
        let maxDays = props.maxDays;
        let start = around;
        let end = around;
        switch (parsedView.value) {
            case 'month':
                start = getStartOfMonth(around);
                end = getEndOfMonth(around);
                maxDays = daysInMonth(start.year, start.month);
                break;
            case 'week':
            case 'week-agenda':
            case 'week-scheduler':
                start = getStartOfWeek(around, props.weekdays, times.today);
                end = getEndOfWeek(start, props.weekdays, times.today);
                maxDays = props.weekdays.length;
                break;
            case 'day':
            case 'scheduler':
            case 'agenda':
                end = moveRelativeDays(copyTimestamp(end), nextDay, maxDays > 1 ? maxDays - 1 : maxDays, props.weekdays);
                end = updateFormatted(end);
                break;
            case 'month-interval':
            case 'month-scheduler':
            case 'month-agenda':
                start = getStartOfMonth(around);
                end = getEndOfMonth(around);
                end = updateFormatted(end);
                maxDays = daysInMonth(start.year, start.month);
                break;
            case 'resource':
                maxDays = 1;
                end = moveRelativeDays(copyTimestamp(end), nextDay, maxDays, props.weekdays);
                end = updateFormatted(end);
                break;
        }
        return { start, end, maxDays };
    });
    return {
        renderValues,
    };
}

const useMoveEmits = ['moved'];
function useMove(props, { parsedView, parsedValue, direction, maxDays, times, emittedValue, emit }) {
    /**
     * Moves the calendar the desired amount. This is based on the 'view'.
     * A month calendar moves by prev/next month
     * A week calendar moves by prev/next week
     * Other considerations are the allowed weekdays; if a day of the week should be displayed (e.g., weekends turned off).
     * @param {number} amount The amount to move (default 1)
     * @fires 'moved' with current Timestamp
     */
    function move(amount = 1) {
        if (amount === 0) {
            emittedValue.value = today();
            return;
        }
        let moved = copyTimestamp(parsedValue.value);
        const lastDayOfMonth = getEndOfMonth(moved);
        const forward = amount > 0;
        const mover = forward ? nextDay : prevDay;
        const limit = forward ? lastDayOfMonth.day : 1; // 1st day of month
        let count = forward ? amount : -amount;
        direction.value = forward ? 'next' : 'prev';
        // Instead of computing dayCount via weekdaySkips,
        // we simply count the allowed weekdays.
        const dayCount = props.weekdays.length;
        while (--count >= 0) {
            switch (parsedView.value) {
                case 'month':
                    // For month view, set to the first (or last) day of the month,
                    // move one day, update the weekday, and adjust until an allowed day is reached.
                    moved.day = limit;
                    moved = mover(moved);
                    moved = updateWeekday(moved);
                    while (!props.weekdays.includes(Number(moved.weekday))) {
                        moved = addToDate(moved, { day: forward ? 1 : -1 });
                    }
                    break;
                case 'week':
                case 'week-agenda':
                case 'week-scheduler':
                    // For week-based views, use relativeDays with allowed weekdays.
                    moved = relativeDays(moved, mover, dayCount, props.weekdays);
                    break;
                case 'day':
                case 'scheduler':
                case 'agenda':
                    // For day views, move a number of days determined by maxDays, taking allowed weekdays into account.
                    moved = relativeDays(moved, mover, maxDays.value, props.weekdays);
                    break;
                case 'month-interval':
                case 'month-agenda':
                case 'month-scheduler':
                    // For these month views, just set to the first or last day then move one day.
                    moved.day = limit;
                    moved = mover(moved);
                    break;
                case 'resource':
                    // For resource view, similar to the day view.
                    moved = relativeDays(moved, mover, maxDays.value, props.weekdays);
                    break;
            }
        }
        // After moving, update weekday, formatted values, day-of-year, and relative data.
        moved = updateWeekday(moved);
        moved = updateFormatted(moved);
        moved = updateDayOfYear(moved);
        moved = updateRelative(moved, times.now);
        emittedValue.value = moved.date;
        emit('moved', moved);
    }
    return {
        move,
    };
}

/**
 * Regular expression to match event listeners starting with 'on'.
 */
const listenerRE = /^on[A-Z]/;
/**
 * Provides computed event listeners from the component instance.
 * @param {ComponentInternalInstance | null} vm - Vue's component instance (defaults to `getCurrentInstance()`).
 * @returns {{ emitListeners: EmitListeners }} - Computed map of event listeners.
 */
function useEmitListeners(vm = vue.getCurrentInstance()) {
    return {
        emitListeners: vue.computed(() => {
            const listeners = {};
            // Ensure vm and vm.vnode are defined before accessing props
            if (vm?.vnode?.props) {
                Object.keys(vm.vnode.props).forEach((key) => {
                    if (listenerRE.test(key)) {
                        listeners[key] = true;
                    }
                });
            }
            return listeners;
        }),
    };
}

/**
 * Creates an accessible focus helper span element.
 * @returns {VNode[]} An array containing a single span VNode.
 */
function useFocusHelper() {
    const spanProps = {
        'aria-hidden': 'true',
        class: 'q-calendar__focus-helper',
    };
    return [vue.h('span', spanProps)];
}

function useButton() {
    function renderButton({ focusable, focusType }, data, slotData) {
        // Ensure the button is focusable based on props
        const isFocusable = focusable && focusType.includes('date');
        return vue.h('button', { ...data, tabindex: isFocusable ? 0 : -1 }, [
            slotData,
            isFocusable && useFocusHelper(),
        ]);
    }
    return { renderButton };
}

const useCheckChangeEmits = ['change'];
function useCheckChange(emit, { days, lastStart, lastEnd }) {
    function checkChange() {
        const dayList = days.value;
        if (dayList.length === 0)
            return false;
        const start = dayList[0].date;
        const end = dayList[dayList.length - 1].date;
        if (!lastStart.value || !lastEnd.value || start !== lastStart.value || end !== lastEnd.value) {
            lastStart.value = start;
            lastEnd.value = end;
            emit('change', { start, end, days: dayList });
            return true;
        }
        return false;
    }
    return { checkChange };
}

/**
 * Utility functions for event handling.
 */
function useEventUtils() {
    /**
     * Checks if the event's keyCode matches any of the specified keyCodes.
     * @param {KeyboardEvent} evt - The keyboard event.
     * @param {number | number[]} keyCodes - The key code or an array of key codes to check against.
     * @returns {boolean} True if the keyCode matches, false otherwise.
     */
    function isKeyCode(evt, keyCodes) {
        return (Array.isArray(keyCodes) ? keyCodes : [keyCodes]).includes(evt.keyCode);
    }
    return {
        isKeyCode,
    };
}

/* global document window */
const { isKeyCode } = useEventUtils();
const useNavigationProps = {
    useNavigation: Boolean,
};
/**
 * Hook for handling keyboard navigation in calendar components.
 * @param props The props related to navigation.
 * @param context Various reactive references used for navigation.
 */
function useNavigation(props, { rootRef, focusRef, focusValue, datesRef, parsedView, emittedValue, direction, times, }) {
    let initialized = false;
    vue.onBeforeUnmount(() => {
        endNavigation();
    });
    vue.watch(() => props.useNavigation, (val) => {
        if (val === true) {
            startNavigation();
        }
        else {
            endNavigation();
        }
    });
    if (props.useNavigation === true) {
        startNavigation();
    }
    function startNavigation() {
        if (initialized)
            return;
        if (document) {
            initialized = true;
            document.addEventListener('keyup', onKeyUp);
            document.addEventListener('keydown', onKeyDown);
        }
    }
    function endNavigation() {
        if (document) {
            document.removeEventListener('keyup', onKeyUp);
            document.removeEventListener('keydown', onKeyDown);
            initialized = false;
        }
    }
    function canNavigate(e) {
        if (!e)
            return false;
        if (document) {
            const el = document.activeElement;
            if (el !== document.body && rootRef.value?.contains(el)) {
                return true;
            }
        }
        return false;
    }
    function tryFocus() {
        let count = 0;
        const interval = window.setInterval(() => {
            const focusElement = datesRef.value[focusRef.value];
            if (focusElement) {
                focusElement.focus();
                if (++count === 50 || document.activeElement === focusElement) {
                    window.clearInterval(interval);
                }
            }
            else {
                window.clearInterval(interval);
            }
        }, 250);
    }
    function onKeyDown(e) {
        if (canNavigate(e) && isKeyCode(e, [33, 34, 35, 36, 37, 38, 39, 40])) {
            e.stopPropagation();
            e.preventDefault();
        }
    }
    function onKeyUp(e) {
        if (canNavigate(e) && isKeyCode(e, [33, 34, 35, 36, 37, 38, 39, 40])) {
            const keyActions = {
                33: onPgUp,
                34: onPgDown,
                35: onEnd,
                36: onHome,
                37: onLeftArrow,
                38: onUpArrow,
                39: onRightArrow,
                40: onDownArrow,
            };
            keyActions[e.keyCode]?.();
        }
    }
    function onUpArrow() {
        let tm = copyTimestamp(focusValue.value);
        if (parsedView.value === 'month') {
            tm = addToDate(tm, { day: -7 });
            if (focusValue.value.month !== tm.month) {
                direction.value = 'prev';
                emittedValue.value = tm.date;
                return;
            }
        }
        else {
            tm = addToDate(tm, { minute: Number(props.intervalMinutes) });
        }
        direction.value = 'prev';
        focusRef.value = tm.date;
    }
    function onDownArrow() {
        let tm = copyTimestamp(focusValue.value);
        if (parsedView.value === 'month') {
            tm = addToDate(tm, { day: 7 });
            if (focusValue.value.month !== tm.month) {
                direction.value = 'next';
                emittedValue.value = tm.date;
                return;
            }
        }
        else {
            tm = addToDate(tm, { minute: Number(props.intervalMinutes) });
        }
        direction.value = 'next';
        focusRef.value = tm.date;
    }
    function onLeftArrow() {
        let tm = copyTimestamp(focusValue.value);
        direction.value = 'prev';
        // Keep moving one day back until the day is allowed.
        do {
            tm = addToDate(tm, { day: -1 });
        } while (!props.weekdays.includes(Number(tm.weekday)));
        focusRef.value = tm.date;
    }
    function onRightArrow() {
        let tm = copyTimestamp(focusValue.value);
        direction.value = 'next';
        // Keep moving one day forward until the day is allowed.
        do {
            tm = addToDate(tm, { day: 1 });
        } while (!props.weekdays.includes(Number(tm.weekday)));
        focusRef.value = tm.date;
    }
    function onPgUp() {
        let tm = copyTimestamp(focusValue.value);
        tm = parsedView.value === 'month' ? addToDate(tm, { month: -1 }) : addToDate(tm, { day: -7 });
        direction.value = 'prev';
        focusRef.value = tm.date;
    }
    function onPgDown() {
        let tm = copyTimestamp(focusValue.value);
        tm = parsedView.value === 'month' ? addToDate(tm, { month: 1 }) : addToDate(tm, { day: 7 });
        direction.value = 'next';
        focusRef.value = tm.date;
    }
    function onHome() {
        let tm = copyTimestamp(focusValue.value);
        // For month view, start at the beginning of the month; for week view, get start of week.
        tm =
            parsedView.value === 'month'
                ? getStartOfMonth(tm)
                : getStartOfWeek(tm, props.weekdays || [], times.today);
        // If the computed start is not an allowed day, move backwards until you hit an allowed day.
        while (!props.weekdays.includes(Number(tm.weekday))) {
            tm = addToDate(tm, { day: -1 });
        }
        focusRef.value = tm.date;
    }
    function onEnd() {
        let tm = copyTimestamp(focusValue.value);
        // For month view, get end of month; for week view, get end of week.
        tm =
            parsedView.value === 'month'
                ? getEndOfMonth(tm)
                : getEndOfWeek(tm, props.weekdays || [], times.today);
        // If the computed end is not an allowed day, move backwards until you hit an allowed day.
        while (!props.weekdays.includes(Number(tm.weekday))) {
            tm = addToDate(tm, { day: -1 });
        }
        focusRef.value = tm.date;
    }
    return {
        startNavigation,
        endNavigation,
        tryFocus,
    };
}

// cellWidth composables
const useCellWidthProps = {
    cellWidth: [Number, String],
};

const { renderButton } = useButton();
var QCalendarTask = vue.defineComponent({
    name: 'QCalendarTask',
    directives: { ResizeObserver: ResizeObserverDirective },
    props: {
        ...useTimesProps,
        ...useNavigationProps,
        ...useCellWidthProps,
        ...useCommonProps,
        ...useTaskProps, // last for any overrides
    },
    emits: [
        'update:model-value',
        'update:model-tasks',
        'update:model-title',
        'update:model-footer',
        'task-expanded',
        ...useCheckChangeEmits,
        ...useMoveEmits,
        ...getRawMouseEvents('-date'),
        ...getRawMouseEvents('-day'),
        ...getRawMouseEvents('-head-day'),
    ],
    setup(props, { slots, emit, expose }) {
        const scrollArea = vue.ref(null), pane = vue.ref(null), direction = vue.ref('next'), startDate = vue.ref(props.modelValue || today()), endDate = vue.ref('0000-00-00'), maxDaysRendered = vue.ref(0), // always 0
        emittedValue = vue.ref(props.modelValue), focusRef = vue.ref(props.modelValue || today()), focusValue = vue.ref(parsed(props.modelValue || today())), datesRef = vue.ref({}), 
        // taskRef = ref(null),
        // weekEventRef = ref([]),
        // weekRef = ref([]),
        // headerColumnRef = ref(null),
        size = vue.reactive({ width: 0, height: 0 }), dragOverHeadDayRef = vue.ref(''), dragOverResource = vue.ref(''), 
        // keep track of last seen start and end dates
        lastStart = vue.ref(null), lastEnd = vue.ref(null);
        vue.watch(() => props.view, () => {
            // reset maxDaysRendered
            maxDaysRendered.value = 0;
        });
        const parsedView = vue.computed(() => {
            if (props.view === 'month') {
                return 'month-interval';
            }
            return props.view;
        });
        const vm = vue.getCurrentInstance();
        if (vm === null) {
            throw new Error('current instance is null');
        }
        // initialize emit listeners
        const { emitListeners } = useEmitListeners(vm);
        const { times, setCurrent, updateCurrent } = useTimes(props);
        // update dates
        updateCurrent();
        setCurrent();
        const { 
        // computed
        parsedStart, 
        // parsedEnd,
        dayFormatter, weekdayFormatter, ariaDateFormatter, 
        // methods
        dayStyleDefault, getRelativeClasses, } = useCommon(props, { startDate, endDate, times });
        // const { isSticky } = useCellWidth(props)
        const parsedValue = vue.computed(() => {
            return parseTimestamp(props.modelValue, times.now) || parsedStart.value || times.today;
        });
        focusValue.value = parsedValue.value;
        focusRef.value = parsedValue.value.date;
        // const computedStyles = computed(() => {
        //   const style: CSSProperties = {}
        //   style.minWidth = computedWidth.value
        //   style.maxWidth = computedWidth.value
        //   style.width = computedWidth.value
        //   return style
        // })
        /// @ts-expect-error fix later
        const { renderValues } = useRenderValues(props, {
            parsedView,
            times,
            parsedValue,
        });
        const { rootRef, __renderCalendar } = useCalendar(props, __renderTask, {
            scrollArea,
            pane,
        });
        const { 
        // computed
        days, parsedStartDate, parsedEndDate,
        // methods
        /// @ts-expect-error fix later
         } = useTask(props, emit, {
            times,
        });
        const { move } = useMove(props, {
            parsedView,
            parsedValue,
            direction,
            maxDays: maxDaysRendered,
            times,
            emittedValue,
            emit,
        });
        const { getDefaultMouseEventHandlers } = useMouseEvents(emit, emitListeners);
        const { checkChange } = useCheckChange(emit, { days, lastStart, lastEnd });
        const { isKeyCode } = useEventUtils();
        /// @ts-expect-error fix later
        const { tryFocus } = useNavigation(props, {
            rootRef,
            focusRef,
            focusValue,
            datesRef,
            parsedView,
            emittedValue,
            direction,
            times,
        });
        // const parsedColumnCount = computed(() => {
        //   return days.value.length
        // })
        // const borderWidth = computed(() => {
        //   if (rootRef.value) {
        //     const calendarBorderWidth = getComputedStyle(rootRef.value).getPropertyValue('--calendar-border')
        //     const parts = calendarBorderWidth.split(' ')
        //     const part = parts.filter(part => part.indexOf('px') > -1)
        //     return parseInt(part[ 0 ], 0)
        //   }
        //   return 0
        // })
        const isSticky = vue.ref(true);
        const parsedCellWidth = vue.computed(() => {
            if (props.cellWidth !== undefined) {
                return parseInt(String(props.cellWidth), 10);
            }
            return 150; // default when not specified
        });
        // const computedWidth = computed(() => {
        //   if (rootRef.value) {
        //     const width = size.width || rootRef.value.getBoundingClientRect().width
        //     if (width && parsedColumnCount.value) {
        //       return (width / parsedColumnCount.value) + 'px'
        //     }
        //   }
        //   return (100 / parsedColumnCount.value) + '%'
        // })
        const isDayFocusable = vue.computed(() => {
            return props.focusable === true && props.focusType.includes('day');
        });
        const isDateFocusable = vue.computed(() => {
            return (props.focusable === true &&
                props.focusType.includes('date') &&
                isDayFocusable.value !== true);
        });
        const isWeekdayFocusable = vue.computed(() => {
            return props.focusable === true && props.focusType.includes('weekday');
        });
        const parsedHeight = vue.computed(() => {
            return parseInt(String(props.dayHeight), 10);
        });
        const parsedMinHeight = vue.computed(() => {
            return parseInt(String(props.dayMinHeight), 10);
        });
        vue.watch([days], checkChange, { deep: true, immediate: true });
        vue.watch(() => props.modelValue, (val, oldVal) => {
            if (emittedValue.value !== val) {
                if (props.animated === true) {
                    const v1 = getDayIdentifier(parsed(val));
                    const v2 = getDayIdentifier(parsed(oldVal));
                    direction.value = v1 >= v2 ? 'next' : 'prev';
                }
                emittedValue.value = val;
            }
            focusRef.value = val;
        });
        vue.watch(emittedValue, (val, oldVal) => {
            if (emittedValue.value !== props.modelValue) {
                if (props.animated === true) {
                    const v1 = getDayIdentifier(parsed(val));
                    const v2 = getDayIdentifier(parsed(oldVal));
                    direction.value = v1 >= v2 ? 'next' : 'prev';
                }
                emit('update:model-value', val);
            }
        });
        vue.watch(focusRef, (val) => {
            if (val) {
                focusValue.value = parseTimestamp(val);
            }
        });
        vue.watch(focusValue, () => {
            if (datesRef.value[focusRef.value]) {
                datesRef.value[focusRef.value].focus();
            }
            else {
                // if focusRef is not in the list of current dates of dateRef,
                // then assume month is changing
                tryFocus();
            }
        });
        vue.onBeforeUpdate(() => {
            datesRef.value = {};
            // weekEventRef.value = []
            // weekRef.value = []
        });
        vue.onMounted(() => {
        });
        // public functions
        function moveToToday() {
            emittedValue.value = today();
        }
        function next(amount = 1) {
            move(amount);
        }
        function prev(amount = 1) {
            move(-amount);
        }
        // private functions
        function __onResize({ width, height }) {
            size.width = width;
            size.height = height;
        }
        function __isActiveDate(day) {
            return day.date === emittedValue.value;
        }
        /**
         * Renders the given day with the associated task
         * @param {Timestamp} day Timestamp representing the day
         * @param {any} task The Task
         * @param {number} taskIndex The task index
         * @returns VNode
         */
        function __renderTaskDay(day, task, taskIndex) {
            const slot = slots.day;
            const styler = props.dayStyle || dayStyleDefault;
            const activeDate = props.noActiveDate !== true && parsedValue.value.date === day.date;
            const dragValue = task[props.taskKey];
            const scope = {
                timestamp: day,
                task,
                taskIndex,
                activeDate,
                droppable: dragOverResource.value === dragValue,
            };
            const width = convertToUnit(parsedCellWidth.value);
            const style = {
                width,
                minWidth: width,
                maxWidth: width,
                ...styler({ scope }),
            };
            const dayClass = typeof props.dayClass === 'function' ? props.dayClass({ scope }) : {};
            // const key = day.date + '-' + task.id
            return vue.h('div', {
                // key,
                // ref: (el) => {
                //   if (isDayFocusable.value === true) {
                //     datesRef.value[ key ] = el
                //   }
                // },
                tabindex: isDayFocusable.value === true ? 0 : -1,
                class: {
                    'q-calendar-task__task--day': true,
                    ...dayClass,
                    ...getRelativeClasses(day),
                    'q-active-date': activeDate === true,
                    'q-calendar__hoverable': props.hoverable === true,
                    'q-calendar__focusable': isDayFocusable.value === true,
                },
                style,
                onFocus: () => {
                    if (isDayFocusable.value === true) ;
                },
                ...getDefaultMouseEventHandlers('-day', (event) => {
                    return { scope, event };
                }),
                onDragenter: (e) => {
                    if (props.dragEnterFunc !== undefined && typeof props.dragEnterFunc === 'function') {
                        props.dragEnterFunc(e, 'day', { scope }) === true
                            ? (dragOverResource.value = dragValue)
                            : (dragOverResource.value = '');
                    }
                },
                onDragover: (e) => {
                    if (props.dragOverFunc !== undefined && typeof props.dragOverFunc === 'function') {
                        props.dragOverFunc(e, 'day', { scope }) === true
                            ? (dragOverResource.value = dragValue)
                            : (dragOverResource.value = '');
                    }
                },
                onDragleave: (e) => {
                    if (props.dragLeaveFunc !== undefined && typeof props.dragLeaveFunc === 'function') {
                        props.dragLeaveFunc(e, 'day', { scope }) === true
                            ? (dragOverResource.value = dragValue)
                            : (dragOverResource.value = '');
                    }
                },
                onDrop: (e) => {
                    if (props.dropFunc !== undefined && typeof props.dropFunc === 'function') {
                        props.dropFunc(e, 'day', { scope }) === true
                            ? (dragOverResource.value = dragValue)
                            : (dragOverResource.value = '');
                    }
                },
            }, [slot && slot({ scope }), useFocusHelper()]);
        }
        function __renderTaskDays(task, taskIndex) {
            return days.value.map((day) => __renderTaskDay(day, task, taskIndex));
        }
        function __renderTaskDaysRow(task, taskIndex) {
            const slot = slots.days;
            const scope = {
                timestamps: days.value,
                days: days.value, // deprecated
                task,
                taskIndex,
                cellWidth: parsedCellWidth.value,
            };
            return vue.h('div', {
                class: 'q-calendar-task__task--days-row',
            }, [__renderTaskDays(task, taskIndex), slot && slot({ scope })]);
        }
        function __renderTaskItem(task, taskIndex, indentLevel = 0, expanded = true) {
            const slot = indentLevel === 0 ? slots.task : slots.subtask;
            const scope = {
                start: parsedStartDate.value,
                end: parsedEndDate.value,
                task,
                taskIndex,
                expanded,
            };
            const width = convertToUnit(props.taskWidth);
            const style = {
                width,
                minWidth: width,
                maxWidth: width,
            };
            const isFocusable = props.focusable === true && props.focusType.includes('task');
            return vue.h('div', {
                class: {
                    'q-calendar-task__task--item': true,
                    'q-calendar__sticky': isSticky.value === true,
                    'q-calendar__hoverable': props.hoverable === true,
                    'q-calendar__focusable': isFocusable === true,
                },
                style,
            }, [
                vue.h('div', {
                    style: {
                        flexDirection: 'column',
                        justifyContent: 'center',
                        alignItems: 'center',
                        width: 10 + 10 * indentLevel + 'px',
                    },
                }, [
                    vue.h('div', {
                        class: {
                            'q-calendar__parent': task.children !== undefined,
                            'q-calendar__parent--expanded': task.children !== undefined && task.expanded === true,
                            'q-calendar__parent--collapsed': task.children !== undefined && task.expanded !== true,
                        },
                        onClick: (e) => {
                            e.stopPropagation();
                            task.expanded = !task.expanded;
                            // emit('update:model-tasks', props.modelTasks)
                            emit('task-expanded', { expanded: task.expanded, scope });
                        },
                    }),
                ]),
                slot && slot({ scope }),
                useFocusHelper(),
            ]);
        }
        function __renderTaskRow(task, taskIndex, indentLevel = 0, expanded = true) {
            const height = task.height !== void 0
                ? convertToUnit(parseInt(task.height, 10))
                : parsedHeight.value > 0
                    ? convertToUnit(parsedHeight.value)
                    : 'auto';
            const minHeight = parsedMinHeight.value > 0 ? convertToUnit(parsedMinHeight.value) : void 0;
            const style = {
                height,
            };
            if (minHeight !== void 0) {
                style.minHeight = minHeight;
            }
            const taskRow = vue.h('div', {
                key: task[props.taskKey] + '-' + taskIndex,
                class: {
                    'q-calendar-task__task': indentLevel === 0,
                    'q-calendar-task__task--section': indentLevel !== 0,
                },
                style,
            }, [
                __renderTaskItem(task, taskIndex, indentLevel, expanded),
                __renderTaskDaysRow(task, taskIndex),
            ]);
            if (task.children !== undefined) {
                return [
                    taskRow,
                    vue.h('div', {
                        class: {
                            'q-calendar__child': true,
                            'q-calendar__child--expanded': expanded === true,
                            'q-calendar__child--collapsed': expanded !== true,
                        },
                    }, [
                        __renderTasks(task.children, indentLevel + 1, expanded === false ? expanded : task.expanded),
                    ]),
                ];
            }
            return [taskRow];
        }
        function __renderTasks(tasks = undefined, indentLevel = 0, expanded = true) {
            if (tasks === undefined) {
                tasks = props.modelTasks;
            }
            return tasks
                .map((task, taskIndex) => {
                return __renderTaskRow(task, taskIndex, indentLevel, task.children !== undefined ? task.expanded : expanded);
            })
                .flat();
        }
        function __renderTasksContainer() {
            return vue.h('div', {
                class: {
                    'q-calendar-task__task--container': true,
                    'q-calendar__sticky': isSticky.value === true,
                },
            }, [__renderTasks()]);
        }
        function __renderFooterTask(task, index) {
            const slot = slots['footer-task'];
            const scope = {
                start: parsedStartDate.value,
                end: parsedEndDate.value,
                footer: task,
                index,
            };
            const width = convertToUnit(props.taskWidth);
            const style = {
                width,
                minWidth: width,
                maxWidth: width,
            };
            return vue.h('div', {
                class: {
                    'q-calendar-task__footer--task': true,
                    'q-calendar__sticky': isSticky.value === true,
                },
                style,
            }, [slot && slot({ scope })]);
        }
        function __renderFooterDay(day, task, index) {
            const slot = slots['footer-day'];
            const scope = {
                timestamp: day,
                footer: task,
                index,
            };
            const width = convertToUnit(parsedCellWidth.value);
            const style = {
                width,
                minWidth: width,
                maxWidth: width,
            };
            const footerDayClass = typeof props.footerDayClass === 'function' ? props.footerDayClass({ scope }) : {};
            return vue.h('div', {
                class: {
                    'q-calendar-task__footer--day': true,
                    ...footerDayClass,
                    ...getRelativeClasses(day),
                    'q-calendar__hoverable': props.hoverable === true,
                    'q-calendar__focusable': isDayFocusable.value === true,
                },
                style,
                // onFocus: (e) => {
                //   if (isDayFocusable.value === true) {
                //     focusRef.value = day.date
                //   }
                // }
            }, [slot && slot({ scope })]);
        }
        function __renderFooterDays(task, index) {
            return vue.h('div', {
                class: 'q-calendar-task__footer--day-wrapper',
            }, [days.value.map((day) => __renderFooterDay(day, task, index))]);
        }
        function __renderFooterRows() {
            const isFocusable = props.focusable === true && props.focusType.includes('task');
            return props.modelFooter.map((task, index) => {
                return vue.h('div', {
                    class: {
                        'q-calendar-task__footer--wrapper': true,
                        'q-calendar__hoverable': props.hoverable === true,
                        'q-calendar__focusable': isFocusable === true,
                    },
                }, {
                    default: () => [__renderFooterTask(task, index), __renderFooterDays(task, index)],
                });
            });
        }
        function __renderFooter() {
            return vue.h('div', {
                class: {
                    'q-calendar-task__footer': true,
                    'q-calendar__sticky': isSticky.value === true,
                },
            }, __renderFooterRows());
        }
        function __renderContainer() {
            return vue.h('div', {
                class: {
                    'q-calendar-task__container': true,
                },
            }, [props.noHeader !== true && __renderHead(), __renderTasksContainer(), __renderFooter()]);
        }
        function __renderHeadTask() {
            const slot = slots['head-tasks'];
            const scope = {
                start: parsedStartDate.value,
                end: parsedEndDate.value,
            };
            const width = convertToUnit(parseInt(String(props.taskWidth), 10));
            const style = {
                width,
                minWidth: width,
                maxWidth: width,
            };
            return vue.h('div', {
                class: {
                    'q-calendar-task__head--tasks': true,
                    'q-calendar__sticky': isSticky.value === true,
                },
                style,
            }, [slot && slot({ scope })]);
        }
        function __renderTitleTask(title, index) {
            const slot = slots['title-task'];
            const width = convertToUnit(parseInt(String(props.taskWidth), 10));
            const style = {
                width,
                minWidth: width,
                maxWidth: width,
            };
            const scope = {
                start: parsedStartDate.value,
                end: parsedEndDate.value,
                cellWidth: width,
                title,
                index,
            };
            return vue.h('div', {
                class: {
                    'q-calendar-task__title--task': true,
                    'q-calendar__sticky': isSticky.value === true,
                },
                style,
            }, [slot && slot({ scope })]);
        }
        function __renderHeadWeekday(day) {
            const slot = slots['head-weekday-label'];
            const activeDate = props.noActiveDate !== true && __isActiveDate(day);
            const scope = {
                activeDate,
                timestamp: day,
                disabled: props.disabledWeekdays
                    ? props.disabledWeekdays.includes(Number(day.weekday))
                    : false,
            };
            const data = {
                class: {
                    'q-calendar-task__head--weekday': true,
                    ['q-calendar__' + props.weekdayAlign]: true,
                    'q-calendar__ellipsis': true,
                },
            };
            return vue.h('div', data, (slot && slot({ scope })) || __renderHeadWeekdayLabel(day, props.shortWeekdayLabel));
        }
        function __renderHeadWeekdayLabel(day, shortWeekdayLabel) {
            const weekdayLabel = weekdayFormatter.value(day, shortWeekdayLabel ||
                (props.weekdayBreakpoints[0] > 0 && parsedCellWidth.value <= props.weekdayBreakpoints[0]));
            return vue.h('span', {
                class: 'q-calendar__ellipsis',
            }, props.weekdayBreakpoints[1] > 0 && parsedCellWidth.value <= props.weekdayBreakpoints[1]
                ? minCharWidth(weekdayLabel, Number(props.minWeekdayLabel))
                : weekdayLabel);
        }
        function __renderHeadDayDate(day) {
            const data = {
                class: {
                    'q-calendar-task__head--date': true,
                    ['q-calendar__' + props.dateAlign]: true,
                },
            };
            return vue.h('div', data, __renderHeadDayBtn(day));
        }
        function __renderHeadDayBtn(day) {
            const activeDate = props.noActiveDate !== true && __isActiveDate(day);
            const dayLabel = dayFormatter.value(day, false);
            const headDayLabelSlot = slots['head-day-label'];
            const headDayButtonSlot = slots['head-day-button'];
            const scope = { dayLabel, timestamp: day, activeDate };
            const key = day.date;
            const data = {
                key,
                tabindex: isDateFocusable.value === true ? 0 : -1,
                class: {
                    'q-calendar-task__head--day__label': true,
                    'q-calendar__button': true,
                    'q-calendar__button--round': props.dateType === 'round',
                    'q-calendar__button--rounded': props.dateType === 'rounded',
                    'q-calendar__button--bordered': day.current === true,
                    'q-calendar__hoverable': props.hoverable === true,
                    'q-calendar__focusable': isDateFocusable.value === true,
                },
                disabled: day.disabled,
                onKeydown: (e) => {
                    if (day.disabled !== true && isKeyCode(e, [13, 32])) {
                        e.stopPropagation();
                        e.preventDefault();
                    }
                },
                onKeyup: (e) => {
                    // allow selection of date via Enter or Space keys
                    if (day.disabled !== true && isKeyCode(e, [13, 32])) {
                        emittedValue.value = day.date;
                        if (emitListeners.value.onClickDate !== undefined) {
                            emit('click-date', { scope });
                        }
                    }
                },
                ...getDefaultMouseEventHandlers('-date', (event, eventName) => {
                    if (eventName === 'click-date' || eventName === 'contextmenu-date') {
                        emittedValue.value = day.date;
                        if (eventName === 'click-date') {
                            event.preventDefault();
                        }
                    }
                    return { scope, event };
                }),
            };
            if (props.noAria !== true) {
                data.ariaLabel = ariaDateFormatter.value(day, false);
            }
            return headDayButtonSlot
                ? headDayButtonSlot({ scope })
                : renderButton(props, data, headDayLabelSlot ? headDayLabelSlot({ scope }) : dayLabel);
        }
        function __renderDateHeader(day) {
            if (props.dateHeader === 'stacked') {
                return [
                    props.noDefaultHeaderText !== true && __renderHeadWeekday(day),
                    props.noDefaultHeaderBtn !== true && __renderHeadDayDate(day),
                ].filter((v) => v !== false);
            }
            else if (props.dateHeader === 'inline') {
                if (props.weekdayAlign === 'left' && props.dateAlign === 'right') {
                    return vue.h('div', {
                        class: 'q-calendar__header--inline',
                    }, [
                        props.noDefaultHeaderText !== true && __renderHeadWeekday(day),
                        props.noDefaultHeaderBtn !== true && __renderHeadDayDate(day),
                    ]);
                }
                else if (props.weekdayAlign === 'right' && props.dateAlign === 'left') {
                    return vue.h('div', {
                        class: 'q-calendar__header--inline',
                    }, [
                        props.noDefaultHeaderText !== true && __renderHeadWeekday(day),
                        props.noDefaultHeaderBtn !== true && __renderHeadDayDate(day),
                    ]);
                }
                else {
                    return vue.h('div', {
                        class: 'q-calendar__header--inline',
                    }, [
                        props.noDefaultHeaderText !== true && __renderHeadWeekday(day),
                        props.noDefaultHeaderBtn !== true && __renderHeadDayDate(day),
                    ]);
                }
            }
            else if (props.dateHeader === 'inverted') {
                if (props.weekdayAlign === 'left' && props.dateAlign === 'right') {
                    return vue.h('div', {
                        class: 'q-calendar__header--inline',
                    }, [
                        props.noDefaultHeaderBtn !== true && __renderHeadDayDate(day),
                        props.noDefaultHeaderText !== true && __renderHeadWeekday(day),
                    ]);
                }
                else if (props.weekdayAlign === 'right' && props.dateAlign === 'left') {
                    return vue.h('div', {
                        class: 'q-calendar__header--inline',
                    }, [
                        props.noDefaultHeaderBtn !== true && __renderHeadDayDate(day),
                        props.noDefaultHeaderText !== true && __renderHeadWeekday(day),
                    ]);
                }
                else {
                    return vue.h('div', {
                        class: 'q-calendar__header--inline',
                    }, [
                        props.noDefaultHeaderBtn !== true && __renderHeadDayDate(day),
                        props.noDefaultHeaderText !== true && __renderHeadWeekday(day),
                    ]);
                }
            }
        }
        /**
         * Renders the given day with the associated task
         * @param {Timestamp} day Timestamp representing the day
         * @param {sting} title The Title
         * @param {number} index The task index
         * @returns VNode
         */
        function __renderTitleDay(day, title, index) {
            const slot = slots['title-day'];
            const width = convertToUnit(parsedCellWidth.value);
            const style = {
                width,
                minWidth: width,
                maxWidth: width,
            };
            const scope = {
                timestamp: day,
                title,
                index,
                cellWidth: parsedCellWidth.value,
            };
            const dayClass = typeof props.dayClass === 'function' ? props.dayClass({ scope }) : {};
            const isFocusable = props.focusable === true && props.focusType.includes('day');
            return vue.h('div', {
                class: {
                    'q-calendar-task__title--day': true,
                    ...dayClass,
                    ...getRelativeClasses(day),
                    'q-calendar__hoverable': props.hoverable === true,
                    'q-calendar__focusable': isFocusable === true,
                },
                style,
            }, [slot && slot({ scope }), useFocusHelper()]);
        }
        function __renderHeadDay(day) {
            const headDaySlot = slots['head-day'];
            const headDateSlot = slots['head-date'];
            const activeDate = props.noActiveDate !== true && __isActiveDate(day);
            const scope = {
                timestamp: day,
                activeDate,
                droppable: (dragOverHeadDayRef.value = day.date),
                disabled: props.disabledWeekdays
                    ? props.disabledWeekdays.includes(Number(day.weekday))
                    : false,
            };
            const styler = props.weekdayStyle || dayStyleDefault;
            const weekdayClass = typeof props.weekdayClass === 'function' ? props.weekdayClass({ scope }) : {};
            const width = convertToUnit(parsedCellWidth.value);
            const style = {
                width,
                minWidth: width,
                maxWidth: width,
                ...styler({ scope }),
            };
            const key = day.date;
            const data = {
                key,
                ref: (el) => {
                    datesRef.value[key] = el;
                },
                tabindex: isWeekdayFocusable.value === true ? 0 : -1,
                class: {
                    'q-calendar-task__head--day': true,
                    ...weekdayClass,
                    ...getRelativeClasses(day),
                    'q-active-date': activeDate,
                    'q-calendar__hoverable': props.hoverable === true,
                    'q-calendar__focusable': isWeekdayFocusable.value === true,
                },
                style,
                onFocus: () => {
                    if (isWeekdayFocusable.value === true) {
                        focusRef.value = key;
                    }
                },
                onKeydown: (e) => {
                    if (day.disabled !== true && isKeyCode(e, [13, 32])) {
                        e.stopPropagation();
                        e.preventDefault();
                    }
                },
                onKeyup: (e) => {
                    // allow selection of date via Enter or Space keys
                    if (day.disabled !== true && isKeyCode(e, [13, 32])) {
                        emittedValue.value = day.date;
                    }
                },
                ...getDefaultMouseEventHandlers('-head-day', (event) => {
                    return { scope, event };
                }),
                onDragenter: (e) => {
                    if (props.dragEnterFunc !== undefined && typeof props.dragEnterFunc === 'function') {
                        props.dragEnterFunc(e, 'head-day', { scope }) === true
                            ? (dragOverHeadDayRef.value = day.date)
                            : (dragOverHeadDayRef.value = '');
                    }
                },
                onDragover: (e) => {
                    if (props.dragOverFunc !== undefined && typeof props.dragOverFunc === 'function') {
                        props.dragOverFunc(e, 'head-day', { scope }) === true
                            ? (dragOverHeadDayRef.value = day.date)
                            : (dragOverHeadDayRef.value = '');
                    }
                },
                onDragleave: (e) => {
                    if (props.dragLeaveFunc !== undefined && typeof props.dragLeaveFunc === 'function') {
                        props.dragLeaveFunc(e, 'head-day', { scope }) === true
                            ? (dragOverHeadDayRef.value = day.date)
                            : (dragOverHeadDayRef.value = '');
                    }
                },
                onDrop: (e) => {
                    if (props.dropFunc !== undefined && typeof props.dropFunc === 'function') {
                        props.dropFunc(e, 'head-day', { scope }) === true
                            ? (dragOverHeadDayRef.value = day.date)
                            : (dragOverHeadDayRef.value = '');
                    }
                },
            };
            return vue.h('div', data, [
                // head-day slot replaces everything below it
                headDaySlot !== undefined && headDaySlot({ scope }),
                headDaySlot === undefined && __renderDateHeader(day),
                headDaySlot === undefined && headDateSlot && headDateSlot({ scope }),
                useFocusHelper(),
            ]);
        }
        function __renderHeadDays() {
            return days.value.map((day) => __renderHeadDay(day));
        }
        function __renderTitleDays(title, index) {
            return days.value.map((day) => __renderTitleDay(day, title, index));
        }
        function __renderHeadDaysRow() {
            return vue.h('div', {
                class: {
                    'q-calendar-task__head--days': true,
                },
            }, [...__renderHeadDays()]);
        }
        function __renderTitleDaysRow(title, index) {
            return vue.h('div', {
                class: {
                    'q-calendar-task__title--days': true,
                },
            }, [...__renderTitleDays(title, index)]);
        }
        // ----
        function __renderHead() {
            return vue.h('div', {
                roll: 'presentation',
                class: {
                    'q-calendar-task__head': true,
                    'q-calendar__sticky': isSticky.value === true,
                },
                style: {},
            }, [
                vue.h('div', {
                    style: {
                        position: 'relative',
                        display: 'flex',
                    },
                }, [__renderHeadTask(), __renderHeadDaysRow()]),
                props.modelTitle.map((title, index) => vue.h('div', {
                    class: 'q-calendar-task__title',
                    style: {
                        position: 'relative',
                        display: 'flex',
                    },
                }, [__renderTitleTask(title, index), __renderTitleDaysRow(title, index)])),
            ]);
        }
        function __renderBody() {
            return vue.h('div', {
                class: 'q-calendar-task__body',
            }, [__renderScrollArea()]);
        }
        function __renderScrollArea() {
            return vue.h('div', {
                ref: scrollArea,
                class: {
                    'q-calendar-task__scroll-area': true,
                    'q-calendar__scroll': true,
                },
            }, [__renderContainer()]);
        }
        function __renderTask() {
            const { start, end } = renderValues.value;
            startDate.value = start.date;
            endDate.value = end.date;
            const hasWidth = size.width > 0;
            const weekly = vue.withDirectives(vue.h('div', {
                key: startDate.value,
                class: 'q-calendar-task',
            }, [hasWidth === true && __renderBody()]), [[ResizeObserverDirective, __onResize]]);
            if (props.animated === true) {
                const transition = 'q-calendar--' +
                    (direction.value === 'prev' ? props.transitionPrev : props.transitionNext);
                return vue.h(vue.Transition, {
                    name: transition,
                    appear: true,
                }, () => weekly);
            }
            return weekly;
        }
        // expose public methods
        expose({
            prev,
            next,
            move,
            moveToToday,
            updateCurrent,
        });
        // Object.assign(vm.proxy, {
        //   prev,
        //   next,
        //   move,
        //   moveToToday,
        //   updateCurrent
        // })
        return () => __renderCalendar();
    },
});

const version = '4.1.2';

exports.DAYS_IN_MONTH = DAYS_IN_MONTH;
exports.DAYS_IN_MONTH_LEAP = DAYS_IN_MONTH_LEAP;
exports.DAYS_IN_MONTH_MAX = DAYS_IN_MONTH_MAX;
exports.DAYS_IN_MONTH_MIN = DAYS_IN_MONTH_MIN;
exports.DAY_MIN = DAY_MIN;
exports.FIRST_HOUR = FIRST_HOUR;
exports.MONTH_MAX = MONTH_MAX;
exports.MONTH_MIN = MONTH_MIN;
exports.PARSE_DATE = PARSE_DATE;
exports.PARSE_DATETIME = PARSE_DATETIME;
exports.PARSE_TIME = PARSE_TIME;
exports.QCalendarTask = QCalendarTask;
exports.TIME_CONSTANTS = TIME_CONSTANTS;
exports.addToDate = addToDate;
exports.compareDate = compareDate;
exports.compareDateTime = compareDateTime;
exports.compareTime = compareTime;
exports.compareTimestamps = compareTimestamps;
exports.convertToUnit = convertToUnit;
exports.copyTimestamp = copyTimestamp;
exports.createDayList = createDayList;
exports.createIntervalList = createIntervalList;
exports.createNativeLocaleFormatter = createNativeLocaleFormatter;
exports.daysBetween = daysBetween;
exports.daysInMonth = daysInMonth;
exports.diffTimestamp = diffTimestamp;
exports.findWeekday = findWeekday;
exports.getDate = getDate;
exports.getDateTime = getDateTime;
exports.getDayIdentifier = getDayIdentifier;
exports.getDayOfYear = getDayOfYear;
exports.getDayTimeIdentifier = getDayTimeIdentifier;
exports.getEndOfMonth = getEndOfMonth;
exports.getEndOfWeek = getEndOfWeek;
exports.getMonthFormatter = getMonthFormatter;
exports.getMonthNames = getMonthNames;
exports.getStartOfMonth = getStartOfMonth;
exports.getStartOfWeek = getStartOfWeek;
exports.getTime = getTime;
exports.getTimeIdentifier = getTimeIdentifier;
exports.getWeekday = getWeekday;
exports.getWeekdayFormatter = getWeekdayFormatter;
exports.getWeekdayNames = getWeekdayNames;
exports.getWorkWeek = getWorkWeek;
exports.indexOf = indexOf;
exports.isBetweenDates = isBetweenDates;
exports.isLeapYear = isLeapYear;
exports.isOverlappingDates = isOverlappingDates;
exports.isToday = isToday;
exports.makeDate = makeDate;
exports.makeDateTime = makeDateTime;
exports.maxTimestamp = maxTimestamp;
exports.minCharWidth = minCharWidth;
exports.minTimestamp = minTimestamp;
exports.moveRelativeDays = moveRelativeDays;
exports.nextDay = nextDay;
exports.padNumber = padNumber;
exports.parseDate = parseDate;
exports.parseTime = parseTime;
exports.parseTimestamp = parseTimestamp;
exports.parsed = parsed;
exports.prevDay = prevDay;
exports.relativeDays = relativeDays;
exports.today = today;
exports.updateDayOfYear = updateDayOfYear;
exports.updateDisabled = updateDisabled;
exports.updateFormatted = updateFormatted;
exports.updateMinutes = updateMinutes;
exports.updateRelative = updateRelative;
exports.updateWeekday = updateWeekday;
exports.updateWorkWeek = updateWorkWeek;
exports.validateNumber = validateNumber;
exports.validateTimestamp = validateTimestamp;
exports.version = version;
exports.weeksBetween = weeksBetween;
